/*************************************************************************
 *
 * ADOBE CONFIDENTIAL
 * __________________
 *
 *  Copyright 2012 Adobe Systems Incorporated
 *  All Rights Reserved.
 *
 * NOTICE:  All information contained herein is, and remains
 * the property of Adobe Systems Incorporated and its suppliers,
 * if any.  The intellectual and technical concepts contained
 * herein are proprietary to Adobe Systems Incorporated and its
 * suppliers and are protected by trade secret or copyright law.
 * Dissemination of this information or reproduction of this material
 * is strictly forbidden unless prior written permission is obtained
 * from Adobe Systems Incorporated.
 **************************************************************************/
package com.adobe.cq.commerce.common;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.text.NumberFormat;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Calendar;
import java.util.Collections;
import java.util.Comparator;
import java.util.Currency;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Locale;
import java.util.Map;
import java.util.Set;
import java.util.UUID;

import javax.jcr.Node;
import javax.jcr.NodeIterator;
import javax.jcr.RepositoryException;
import javax.jcr.Session;
import javax.jcr.query.Query;
import javax.servlet.http.Cookie;

import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.collections.Predicate;
import org.apache.commons.lang.StringUtils;
import org.apache.jackrabbit.api.JackrabbitSession;
import org.apache.jackrabbit.api.security.user.Authorizable;
import org.apache.jackrabbit.api.security.user.UserManager;
import org.apache.jackrabbit.util.ISO9075;
import org.apache.sling.api.SlingHttpServletRequest;
import org.apache.sling.api.SlingHttpServletResponse;
import org.apache.sling.api.resource.Resource;
import org.apache.sling.api.resource.ResourceResolver;
import org.apache.sling.api.resource.ValueMap;
import org.apache.sling.api.wrappers.SlingHttpServletResponseWrapper;
import org.apache.sling.api.wrappers.ValueMapDecorator;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.adobe.cq.commerce.api.CommerceConstants;
import com.adobe.cq.commerce.api.CommerceException;
import com.adobe.cq.commerce.api.CommerceSession;
import com.adobe.cq.commerce.api.PaymentMethod;
import com.adobe.cq.commerce.api.PlacedOrder;
import com.adobe.cq.commerce.api.PlacedOrderResult;
import com.adobe.cq.commerce.api.PriceInfo;
import com.adobe.cq.commerce.api.Product;
import com.adobe.cq.commerce.api.ShippingMethod;
import com.adobe.cq.commerce.api.promotion.Promotion;
import com.adobe.cq.commerce.api.promotion.PromotionHandler;
import com.adobe.cq.commerce.api.promotion.PromotionInfo;
import com.adobe.cq.commerce.api.promotion.PromotionInfo.PromotionStatus;
import com.adobe.cq.commerce.api.promotion.PromotionManager;
import com.adobe.cq.commerce.api.promotion.Voucher;
import com.adobe.cq.commerce.api.promotion.VoucherInfo;
import com.adobe.cq.commerce.api.smartlist.SmartListManager;
import com.adobe.cq.commerce.common.promotion.AbstractJcrVoucher;
import com.adobe.cq.commerce.impl.promotion.JcrPromotionImpl;
import com.adobe.cq.commerce.impl.promotion.JcrVoucherImpl;
import com.adobe.granite.security.user.UserProperties;
import com.adobe.granite.security.user.UserPropertiesManager;
import com.day.cq.commons.Language;
import com.day.cq.commons.jcr.JcrUtil;
import com.day.cq.i18n.I18n;
import com.day.cq.personalization.ContextSessionPersistence;
import com.day.cq.personalization.UserPropertiesUtil;

import aQute.bnd.annotation.ConsumerType;


/**
 * This class is meant to be used as a base class for a CommerceSession implementation built
 * on top of a JCR repository.  GeoCommerceSessionImpl is an example.
 *
 * Parts of this class form a reference implementation.  That is, they are meant to show what
 * would be needed in an actual implementation, but aren't in themselves likely to be sufficient
 * for real-world cases.
 */
@ConsumerType
public class AbstractJcrCommerceSession implements CommerceSession {
    protected static final Logger log = LoggerFactory.getLogger(AbstractJcrCommerceSession.class);

    protected SlingHttpServletRequest request;
    protected SlingHttpServletResponse response;
    protected Resource resource;
    protected ResourceResolver resolver;
    protected AbstractJcrCommerceService commerceService;

    protected Locale locale = Locale.US;
    protected Locale userLocale = null;

    protected String PN_UNIT_PRICE = "price";
    protected String PN_ORDER_ID = "orderId";
    protected RoundingMode roundingMode = RoundingMode.HALF_UP;
    protected BigDecimal PRODUCT_TAX_RATE = new BigDecimal("0.06");
    protected BigDecimal SHIPPING_TAX_RATE = BigDecimal.ZERO;
    private String orderId;

    /**
     * The maximum size of the CommercePersistence cookie.
     * When this size is exceeded the cookie and subsequently the shopping cart is not updated.
     * The minimum cookie size that browsers are required to support is 4KB but there is a browser specific
     * overhead for a cookie which limits the length of useful content that can be stored in a cookie.
     * The value bellow has been chosen based on these considerations.
     */
    protected static final int COOKIE_SIZE_LIMIT = 4050;
    protected static final String PN_COMMERCE_PERSISTENCE_OVERFLOW = "cpo";
    protected static final int COOKIE_MAX_AGE_SECONDS = 3600 * 24 * 31;

    protected static final String ORDERS_BASE_PATH = "/etc/commerce/orders/";
    protected static final String ORDERS_PATH_DATE_TEMPLATE = "yyyy/MM/dd";
    protected static final String ORDER_NAME = "order";

    protected static final String USER_ORDERS_PATH = "/commerce/orders/";
    protected static final String USER_ORDERS_DATE_TEMPLATE = "'order'-yyyy-MMM-dd";

    protected List<CartEntry> cart = new ArrayList<CartEntry>();

    protected List<Voucher> vouchers = new ArrayList<Voucher>();
    protected List<Promotion> promotions = new ArrayList<Promotion>();

    protected Map<String, String> orderDetails = new HashMap<String, String>();

    protected List<PriceInfo> prices;

    public AbstractJcrCommerceSession(AbstractJcrCommerceService commerceService,
                                      SlingHttpServletRequest request,
                                      SlingHttpServletResponse response,
                                      Resource resource) throws CommerceException {
        this.request = request;
        this.response = response;
        this.resource = resource;
        this.resolver = resource.getResourceResolver();
        this.commerceService = commerceService;

        Language lang = commerceService.serviceContext().languageManager.getCqLanguage(resource);
        if (lang != null && lang.getLocale().getCountry().length() > 0) {
            locale = lang.getLocale();
            loadCart();
        } else {
            loadCart();
            if (resource.getPath().startsWith("/content")) {
                log.debug("Unable to extract locale from page {}, falling back to default locale {}.", resource.getPath(), locale);
            } else {
                // Some resources, such as clientcontext store initializers and segmentation definitions,
                // are used for multiple languages/locales.  For empty carts we probably don't much care
                // what the locale is, and for non-empty carts we can infer the locale from the first item.
                if (cart.size() > 0) {
                    Resource firstProductPage = resolver.getResource(cart.get(0).getProduct().getPagePath());
                    if (firstProductPage != null) {
                        lang = commerceService.serviceContext().languageManager.getCqLanguage(firstProductPage);
                        if (lang != null && lang.getLocale().getCountry().length() > 0) {
                            locale = lang.getLocale();
                            calcOrder(); // re-calculate with the new locale
                        }
                    }
                }
                // If we didn't find a product page then we're going to go with the default locale, but
                // there's not much point in warning the author about something they can't fix, so we
                // suppress the log entry in this case.
                // log.warn("Unable to extract locale from page {}, falling back to default locale {}.", resource.getPath(), locale);
            }
        }
    }

    /**
     * Package internal constructor for testing.
     */
    AbstractJcrCommerceSession(ResourceResolver resolver) {
        this.resolver = resolver;
    }

    /**
     * Load a product into the cart.
     */
    private void loadProduct(String productPath, String quantityString, Map<String, Object> otherProperties) {
	}

    /**
     * Load a voucher into the cart.
     */
    private void loadVoucher(String voucherPath) {
	}

    /**
     * Load a promotion into the cart.
     */
    private void loadPromotion(String promotionPath) {
	}

    /**
     * Load the session state (including the cart entries, the order details, and any applied vouchers and promotions).
     *
     * <p>This implementation uses cookies.  In most cases concrete implementations will want to provide their own storage
     * architecture.</p>
     */
    protected void loadCart() throws CommerceException {
	}

    /**
     * Save the session state (including the cart entries, the order details, and any applied vouchers and promotions).
     *
     * <p>This implementation uses cookies, although a commented-out JCR storage mechanism is also provided.  In most
     * cases concrete implementations will want to provide their own storage architecture.</p>
     */
    protected void saveCart() throws CommerceException {
	}

    /**
     * A helper routine for writing cookies (and dealing with cookie length restrictions).
     */
    private void saveCommerceCookie(Map<String, Map<String, String>> stores) {
	}

    /**
     * Returns true if an overflow is detected on the commerce persistence cookie.
     */
    public static boolean hasCookieOverflow(SlingHttpServletRequest request, SlingHttpServletResponse response) {
		return false;
	}

    /**
     * Logout is a NO-OP in this implementation.
     *
     * <p>To be overridden by concrete implementations requiring logout semantics (such as those employing
     * session state).</p>
     */
    @Override
    public void logout() throws CommerceException {
	}

    /**
     * Get the current locale.  Will be the <code>userLocale</code> if set, otherwise the <code>locale</code>.
     */
    protected Locale getLocale() {
		return null;
	}

    /**
     * Set the userLocale (which, if non-null, overrides the default locale).
     */
    @Override
    public void setUserLocale(Locale locale) {
	}

    @Override
    public Locale getUserLocale() {
		return null;
	}

    /**
     * The base implementation does not support session- or order-specific destinations, so this
     * is a straight pass-through to the commerce service.
     */
    @Override
    public List<String> getAvailableCountries() throws CommerceException {
		return null;
	}

    /**
     * The base implementation does not support session- or order-specific shipping methods, so this
     * is a straight pass-through to the commerce service.
     */
    @Override
    public List<ShippingMethod> getAvailableShippingMethods() throws CommerceException {
		return null;
	}

    /**
     * The base implementation does not support session- or order-specific payment methods, so this
     * is a straight pass-through to the commerce service.
     */
    @Override
    public List<PaymentMethod> getAvailablePaymentMethods() throws CommerceException {
		return null;
	}

    @Override
    public List<PriceInfo> getProductPriceInfo(Product product) throws CommerceException {
		return null;
	}

    /**
     * A simple product pricing architecture supporting single-currency pricing and a fixed tax rate.
     *
     * <p>Note: most concrete implementations will want to override this to supply their own pricing
     * architecture.</p>
     */
    @Override
    public List<PriceInfo> getProductPriceInfo(Product product, Predicate filter) throws CommerceException {
		return null;
	}

    @Override
    public String getProductPrice(Product product) throws CommerceException {
		return null;
	}

    @Override
    public String getProductPrice(Product product, Predicate filter) throws CommerceException {
		return null;
	}

    @Override
    public int getCartEntryCount() {
		return 0;
	}

    @Override
    public List<CartEntry> getCartEntries() {
		return null;
	}

    @Override
    /**
     * For a discussion of the classification and usage of prices, see {@link PriceInfo}.
     */
    public List<PriceInfo> getCartPriceInfo(Predicate filter) {
		return null;
	}

    @Override
    public String getCartPrice(Predicate filter) throws CommerceException {
		return null;
	}

    @Override
    public void addCartEntry(Product product, int quantity) throws CommerceException {
	}

    @Override
    public void addCartEntry(Product product, int quantity, Map<String, Object> properties) throws CommerceException {
	}

    /**
     * Increment the quantity if the product already exists in the cart; otherwise add it.  Any properties
     * associated with the product are updated in either case.
     *
     * <p>NB: recalculates the entry, but not the cart.</p>
     *
     * @param product
     * @param quantity
     * @param properties
     * @throws CommerceException
     */
    protected void doAddCartEntry(Product product, int quantity, Map<String, Object> properties) throws CommerceException {
	}

    @Override
    public void modifyCartEntry(int entryNumber, int quantity) throws CommerceException {
	}

    @Override
    public void modifyCartEntry(int entryNumber, Map<String, Object> delta) throws CommerceException {
	}

    /**
     * Updates the quantity and properties of a cart entry.
     *
     * <p>NB: the cart entry is recalculated, but not the cart.</p>
     *
     * @param entryNumber
     * @param quantity
     * @param delta
     * @throws CommerceException
     */
    protected void doModifyCartEntry(int entryNumber, Integer quantity, Map<String, Object> delta) throws CommerceException {
	}

    @Override
    public void deleteCartEntry(int entryNumber) throws CommerceException {
	}

    /**
     * Recalculates the pricing info of an entry in the cart.
     * @param index The index of the entry.
     * @throws CommerceException
     */
    public void calcEntry(int index) throws CommerceException {
	}

    /**
     * A simple entry pricing architecture in which a single currency unitPrice is stored in the
     * Product itself, and there is a single, fixed tax rate.
     *
     * <p>Note: most concrete implementations will want to override this to supply their own pricing
     * architecture.</p>

     * @param entry The line item in the cart to calculate.
     * @param discount A discount to apply to the total amount.
     * @param locale The locale (currency) to recalculate.
     */
    protected void doCalcEntry(DefaultJcrCartEntry entry, BigDecimal discount, Locale locale) throws CommerceException {
	}

    /**
     * A helper routine which updates the order prices with a new {@link PriceInfo}.  If all the
     * priceInfo's types match an existing entry, the entry will be updated.
     *
     * <p>Note: automatically add the priceInfo's currencyCode as an additional tag to ease
     * filtering on currency.</p>
     *
     * @param priceInfo A {@link PriceInfo} containing the amount and currency.
     * @param types Classifiers indicating the usage of the PriceInfo (ie: "UNIT", "PRE_TAX").
     */
    protected void setPrice(PriceInfo priceInfo, String... types) {
	}

    /**
     * A simple cart pricing architecture with no volume discount structure, shipping calculations,
     * etc., and a single currency.  Just add up the cart entries.
     *
     * <p>Note: most concrete implementations will want to override this to supply their own pricing
     * architecture.</p>
     */
    protected void calcCart() {
	}

    /**
     * Returns the list of currently active promotions (including those fired by vouchers).
     * @return List of active promotions
     */
    public List<Promotion> getActivePromotions() {
		return null;
	}

    @Override
    public boolean supportsClientsidePromotionResolution() {
		return false;
	}

    /**
     * Adds a client-side-resolved promotion to the current session.
     * @param path A string uniquely identifying the promotion (the path for internal {@link Promotion}s;
     *             an id for external ones).)
     * @throws CommerceException
     */
    @Override
    public void addPromotion(String path) throws CommerceException {
	}

    @Override
    /**
     * Removes a client-side-resolved promotion form the current session.
     */
    public void removePromotion(String path) throws CommerceException {
	}

    /**
     * A CQ-centric implementation which assumes all promotion resolution is done on the client.  The list
     * of promotions is therefore exactly what was set by calls to add/removePromotion().
     *
     * <p>To be overridden by concrete implementations wishing to provide external-commerce-engine-resolved
     * promotions (either exclusively or in addition to CQ-resolved promotions).</p>
     */
    @Override
    public List<PromotionInfo> getPromotions() throws CommerceException {
		return null;
	}

    /**
     * Calculate the shipping amount.
     *
     * <p>Note: to be overridden by concrete implementation.</p>
     */
    protected BigDecimal getShipping(String method) {
		return null;
	}

    /**
     * A simple, single-currency order pricing architecture.
     *
     * <p>Note: most concrete implementations will want to override this to supply their own pricing
     * architecture.</p>
     */
    protected void calcOrder() throws CommerceException {
	}

    /**
     * Return a list of vouchers that were added to the cart via {@link #addVoucher(String)}, complete with
     * run-time details such as shopper messages and applied discount information.
     */
    @Override
    public List<VoucherInfo> getVoucherInfos() throws CommerceException {
		return null;
	}

    /**
     * Add a {@link Voucher}, identified by its code, to the cart.  Invalid vouchers will result in a
     * {@link CommerceException} with an internationalized message regarding the validity status.
     */
    @Override
    public void addVoucher(String code) throws CommerceException {
	}

    /**
     * Remove all {@link Voucher}s from the cart which match a particular voucher code.
     */
    @Override
    public void removeVoucher(String code) throws CommerceException {
	}

    /**
     * Return an ID uniquely identifying the current order.
     */
    @Override
    public String getOrderId() throws CommerceException {
		return null;
	}

    /**
     * Handle order updates.  This implementation assumes any relevant predicates will come in as prefixes
     * on the individual map keys (eg: "billing.street"), which is most friendly to CQ forms processing.
     *
     * <p>Will be called multiple times for multi-page checkouts.</p>
     *
     * <p>Much of the payment info is sensitive.  Some of it, such as the CCV, should not be stored at all,
     * and other items, such as the primary-account-number should only be stored if encrypted.</p>
     *
     * <p>See https://www.pcisecuritystandards.org/documents/PCI%20SSC%20Quick%20Reference%20Guide.pdf for
     * further info.</p>
     *
     * <p>This implementation tokenizes payment info.  For this reason, all payment info must be updated in
     * a single call (although it does not need to the the only call).  Tokenization is left to the
     * concrete implementation (see {@link #tokenizePaymentInfo(Map)} for more details).</p>
     */
    protected void doUpdateOrderDetails(Map<String, String> delta) throws CommerceException {
	}

    /**
     * This implementation assumes the payment option will be keyed with {@link CommerceConstants#PAYMENT_OPTION},
     * and that the value associated with that key will be the path to a resource adaptable to {@link PaymentMethod}.
     *
     * It should be overridden to support other options/implementations.
     */
    protected PaymentMethod getPaymentMethod(Map<String, String> delta) {
		return null;
	}

    @Override
    public void updateOrder(Map<String, Object> delta) throws CommerceException {
	}

    /**
     * Handle predicate-based order updates.  This implementation assumes a flat order property space,
     * and simply prefixes any keys with the predicate name.
     */
    protected void doUpdateOrderDetails(Map<String, Object> delta, String predicate) throws CommerceException {
	}

    @Override
    public void updateOrderDetails(Map<String, Object> details, String predicate) throws CommerceException {
	}

    @Override
    public Map<String, String> getOrderDetails() throws CommerceException {
		return null;
	}

    @Override
    public Map<String, Object> getOrder() throws CommerceException {
		return null;
	}

    /**
     * This implementation uses a flat internal order property storage with (optionally) prefixed
     * property keys.
     *
     * <p>Predicate-based queries match on prefix, and return "naked" keys (ie: with the prefixes
     * stripped off).</p>
     */
    @Override
    public Map<String, Object> getOrderDetails(String predicate) throws CommerceException {
		return null;
	}

    /**
     * Returns a token which uniquely identifies an in-progress payment.
     *
     * <p>To be overridden by concrete implementation.</p>
     * @param paymentDetails The payment details, such as credit card info, PayPal account info, etc.
     * @return A token uniquely identifying the payment.
     */
    protected String tokenizePaymentInfo(Map<String, String> paymentDetails) throws CommerceException {
		return null;
	}

    /**
     * Update any order details and then save the cart and order details to the repo.
     * Clear the shopping cart, and initiate any required order processing.
     */
    protected void doPlaceOrder(Map<String, String> orderDetailsDelta) throws CommerceException {
	}

    /**
     * Perform application-specific steps to process an order (such as payment processing and fulfillment).
     *
     * <p>Some applications might delegate order processing to a workflow, in which case this method might
     * remain a NO-OP.</p>
     */
    protected void initiateOrderProcessing(String orderPath) throws CommerceException {
	}

    /**
     * Return a string describing the current status of a placed order.
     *
     * <p>To be overridden by concrete implementation.</p>
     */
    protected String getOrderStatus(String orderId) throws CommerceException {
		return null;
	}

    @Override
    public void placeOrder(Map<String, Object> delta) throws CommerceException {
	}

    /**
     * Create the orders storage location if it doesn't yet exist.  Make sure it's locked down by the
     * service session.
     */
    protected void initializeOrderStorage(Session session) throws RepositoryException {
	}

    /**
     * Saves a copy of a placed order for the vendor in /etc/commerce/orders.
     *
     * <p>To be overridden by concrete implementations which have more specific requirements on the details
     * saved, or the location of saved details.</p>
     */
    protected String savePlacedOrderVendorRecord() throws CommerceException {
		return null;
	}

    /**
     * Saves a copy of a placed order for the shopper in ~/commerce/orders.
     *
     * <p>To be overridden by concrete implementations which have more specific requirements on the details
     * saved, or the location of saved details.</p>
     */
    protected void savePlacedOrderShopperRecord() throws CommerceException {
	}

    /**
     * Writes a copy of an order (including cart line items, order details, applied promotions and applied vouchers)
     * to a node in the repository.
     */
    protected void writeOrder(Node orderNode, Calendar placedTime, Session session) throws CommerceException, RepositoryException {
	}

    /**
     * Serializes a {@link CartEntry} to a <code>String</code> of the form <code>"product_path;quantity;property1_name=property1_value\fproperty2_name=property2_value\f..."</code>.
     */
    protected String serializeCartEntry(CartEntry entry) throws CommerceException {
		return null;
	}

    /**
     * Constructs a CartEntry object from a String created by {@link #serializeCartEntry(com.adobe.cq.commerce.api.CommerceSession.CartEntry)}.
     *
     * @param str the serialized cart entry
     * @param index the cart entry index
     * @return the CartEntry instance
     * @throws CommerceException
     */
    protected CartEntry deserializeCartEntry(String str, int index) throws CommerceException {
		return null;
	}

    /**
     * Serializes an order property to a <code>String</code> of the form <code>"property_name=property_value"</code>.
     *
     * <p><b>NB: sensitive information is stripped during serialization.</b></p>
     */
    protected String serializeOrderDetail(String key, String value) throws CommerceException {
		return null;
	}

    /**
     * Serializes a valid {@link VoucherInfo} to a <code>String</code> of the form <code>"code;path;message"</code>.
     * Invalid vouchers return <code>null</code>.
     */
    protected String serializeVoucherInfo(VoucherInfo voucherInfo) {
		return null;
	}

    /**
     * Serializes a fired {@link PromotionInfo} to a <code>String</code> of the form
     * <code>"promo_path;cart_entry_index;message"</code>.  Un-fired promotions return <code>null</code>.
     */
    protected String serializePromotionInfo(PromotionInfo promotionInfo) {
		return null;
	}

    /**
     * Returns a list of placed orders for the current shopper.
     * @param predicate An optional implementation-specific predicate name.
     */
    @Override
    public PlacedOrderResult getPlacedOrders(String predicate, int pageNumber, int pageSize, String sortId) throws CommerceException {
		return null;
	}

    /**
     * Return a {@link Predicate} (identified by a String key) for {@link PlacedOrder} filtering.
     *
     * <p>To be overridden by concrete implementation.  Implementations should support at least the key
     * {@link CommerceConstants#OPEN_ORDERS_PREDICATE}.</p>
     */
    protected Predicate getPredicate(String predicateName) {
		return null;
	}

    /**
     * Instantiates a new placed order.  Override this method to supply custom extensions of {@link DefaultJcrPlacedOrder}.
     * @param orderId The unique identifier of the order.
     * @return
     */
    public DefaultJcrPlacedOrder newPlacedOrderImpl(String orderId) {
		return null;
	}

    @Override
    public PlacedOrder getPlacedOrder(String orderId) throws CommerceException {
		return null;
	}

    /**
     * Default implementation assumes a repository-based smart lists and the corresponding smart list manager. Override
     * this method to supply custom implementation of {@link SmartListManager}.
     */
    @Override
    public SmartListManager getSmartListManager() {
		return null;
	}

    /*
     * ==================================================================================================
     * Deprecated methods.  For backwards-compatibility only.
     * ==================================================================================================
     */

    @Deprecated // since 5.6
    protected NumberFormat formatter = NumberFormat.getCurrencyInstance(locale);

    @Deprecated // since 5.6
    public String getPriceInfo(Product product) throws CommerceException {
		return null;
	}

    @Deprecated // since 5.6
    public String getCartPreTaxPrice() throws CommerceException {
		return null;
	}
    @Deprecated // since 5.6
    public String getCartTax() throws CommerceException {
		return null;
	}
    @Deprecated // since 5.6
    public String getCartTotalPrice() throws CommerceException {
		return null;
	}
    @Deprecated // since 5.6
    public String getOrderShipping() throws CommerceException {
		return null;
	}
    @Deprecated // since 5.6
    public String getOrderTotalTax() throws CommerceException {
		return null;
	}
    @Deprecated // since 5.6
    public String getOrderTotalPrice() throws CommerceException {
		return null;
	}
    @Deprecated // since 5.6
    public String getShippingPrice(String method) {
		return null;
	}

    /**
     * A simple, single-currency, single-locale price formatter.
     * @deprecated since 5.6, use {@link PriceInfo} instead
     */
    @Deprecated // since 5.6
    protected String formatPrice(BigDecimal price) {
		return null;
	}

    /**
     * @deprecated since 6.0; use {@link #getVoucherInfos()} instead
     */
    @Deprecated
    public List<Voucher> getVouchers() throws CommerceException {
		return null;
	}

    @Deprecated // since 5.6; use updateOrder()
    public void updateOrderDetails(Map<String, String> delta) throws CommerceException {
	}

    @Deprecated // since 5.6; use placeOrder()
    public void submitOrder(Map<String, String> orderDetailsDelta) throws CommerceException {
	}

    /**
     * Instantiates a new cart entry.  Override this method to supply custom extensions of {@link DefaultJcrCartEntry}.
     *
     * @param index The index of the entry with the current cart or PlacedOrder.
     * @param product The product the new entry represents.
     * @param quantity The quantity.
     *
     * @deprecated since 6.1; use {@link AbstractJcrCommerceService#newCartEntryImpl(int, com.adobe.cq.commerce.api.Product, int)} instead.
     */
    @Deprecated
    public DefaultJcrCartEntry newCartEntryImpl(int index, Product product, int quantity) {
		return null;
	}

    /**
     * Increment the quantity if the product already exists in the cart; otherwise add it.
     *
     * <p>NB: re-calculates the entry, but not the cart.</p>
     *
     * @deprecated since 6.1; use {@link #doAddCartEntry(com.adobe.cq.commerce.api.Product, int, Map)} instead.
     */
    @Deprecated
    protected void doAddCartEntry(Product product, int quantity) throws CommerceException {
	}

}
