/*************************************************************************
 *
 * ADOBE CONFIDENTIAL
 * __________________
 *
 *  Copyright 2012 Adobe Systems Incorporated
 *  All Rights Reserved.
 *
 * NOTICE:  All information contained herein is, and remains
 * the property of Adobe Systems Incorporated and its suppliers,
 * if any.  The intellectual and technical concepts contained
 * herein are proprietary to Adobe Systems Incorporated and its
 * suppliers and are protected by trade secret or copyright law.
 * Dissemination of this information or reproduction of this material
 * is strictly forbidden unless prior written permission is obtained
 * from Adobe Systems Incorporated.
 **************************************************************************/
package com.day.image;

import java.awt.Color;
import java.awt.Rectangle;
import java.awt.geom.Point2D;
import java.awt.geom.Rectangle2D;
import java.awt.image.BufferedImage;

/**
 * The <code>DistortOp</code> class implements the distortion operation on a
 * <code>BufferedImage</code>. The rectangular image is distorted such that the
 * orignal corners of the image are at four new coordinate locations. This
 * distortion is generally not linear and requires quite some calculation.
 * <p>
 * <em><strong>
 * NOTE: This operation works uses copies of the full image to caclulate the
 * new image. This takes an amount of memory relative to the size of the image.
 * To calculate the needed space let srcWidth and srcHeight be the width and
 * height, resp., of the source and dstWidth and dstHeight be the width and
 * height, resp., of the destination image. Then the the source image needs :
 * srcWidth * srcHeight * 4 channels * 4 bytes per sample bytes and the
 * destination image needs dstWidth * dstHeight * 4 channels * 4 bytes.
 * </strong></em>
 *
 * @version $Revision$
 * @author fmeschbe
 * @since coati
 * @audience wad
 */
public class DistortOp extends AbstractBufferedImageOp {

    /** The coordinates */
    private final double x00;
    private final double x01;
    private final double x10;
    private final double x11;
    private final double y00;
    private final double y01;
    private final double y10;
    private final double y11;

    /** Whether to crop the result to the current size */
    private final boolean crop;

    /** the background color for unlit pixels */
    private final Color bgColor;

    /**
     * Creates the <code>DistortOp</code> object by supplying the coordinates
     * of the new corners of the image, where the result is not cropped and
     * pixels not filled with any part of the distorted image is replaced with
     * black.
     *
     * @param coords 4x2 matrix describing an array of 4 scale factors by which
     *      to transform each corner of an image distorted image in order
     *      top/left, top/right, bottom/left, bottom/right.
     *
     * @throws IllegalArgumentException if not enough - namely four - coordinate
     * 		pairs are supplied to the constructor.
     * @throws NullPointerException if the coords parameter is <code>null</code>
     * 		or if any of the coordinates is <code>null</code>.
     */
    public DistortOp(float[][] coords) {
	this(coords, false, Color.black);
    }

    /**
     * Creates the <code>DistortOp</code> object by supplying the coordinates
     * of the new corners of the image.
     *
     * @param coords 4x2 matrix describing an array of 4 scale factors by which
     *      to transform each corner of an image distorted image in order
     *      top/left, top/right, bottom/left, bottom/right.
     * @param crop <code>true</code> if the result should be cropped to the
     *      size of the source image.
     * @param bgColor The color to use for pixels not covered by the distorted
     *      part of the image.
     *
     * @throws IllegalArgumentException if not enough - namely four - coordinate
     * 		pairs are supplied to the constructor.
     * @throws NullPointerException if the coords parameter is <code>null</code>
     * 		or if any of the coordinates is <code>null</code>.
     */
    public DistortOp(float[][] coords, boolean crop, Color bgColor) {
	super(null);

	// check arguments
	if (coords == null) {
	    throw new NullPointerException("coords");
	}
	if (coords.length < 3) {
	    throw new IllegalArgumentException("need 4 coordinate pairs");
	}
	for (int i=0; i < 4; i++) {
	    if (coords[i] == null) {
		throw new NullPointerException("coords[" + i + "]");
	    }
	    if (coords[i].length < 2) {
		throw new IllegalArgumentException("missing value on coords["
		    + i + "]");
	    }
	}

	// assign values
	this.x00 = coords[0][0];
	this.y00 = coords[0][1];
	this.x10 = coords[1][0];
	this.y10 = coords[1][1];
	this.x01 = coords[2][0];
	this.y01 = coords[2][1];
	this.x11 = coords[3][0];
	this.y11 = coords[3][1];
	this.crop = crop;
	this.bgColor = bgColor;
    }

    //----------- BufferedImageOp interface ------------------------------------

    /**
     * Returns the bounding box of the filtered destination image.
     * The IllegalArgumentException may be thrown if the source
     * image is incompatible with the types of images allowed
     * by the class implementing this filter.
     */
    public Rectangle2D getBounds2D(BufferedImage src) {
		return null;
	}

    /**
     * Returns the location of the destination point given a
     * point in the source image.  If dstPt is non-null, it
     * will be used to hold the return value.
     */
    public Point2D getPoint2D(Point2D srcPt, Point2D dstPt) {
		return null;
	}

    //---------- protected -----------------------------------------------------

    protected void doFilter(BufferedImage src, BufferedImage dst) {
	}
}
