/*************************************************************************
 *
 * ADOBE CONFIDENTIAL
 * __________________
 *
 *  Copyright 2012 Adobe Systems Incorporated
 *  All Rights Reserved.
 *
 * NOTICE:  All information contained herein is, and remains
 * the property of Adobe Systems Incorporated and its suppliers,
 * if any.  The intellectual and technical concepts contained
 * herein are proprietary to Adobe Systems Incorporated and its
 * suppliers and are protected by trade secret or copyright law.
 * Dissemination of this information or reproduction of this material
 * is strictly forbidden unless prior written permission is obtained
 * from Adobe Systems Incorporated.
 **************************************************************************/
package com.day.image;

import java.awt.AlphaComposite;
import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.RenderingHints;
import java.awt.image.BufferedImage;
import java.awt.image.ColorModel;
import java.awt.image.IndexColorModel;
import java.awt.image.Raster;
import java.awt.image.WritableRaster;

/**
 * The <code>DitherOp</code> provides the dithering capability for reducing
 * colors of an image to any number of colors. The real number of colors at the
 * end of color reduction depends on the color profile of the image under
 * reduction.
 * <p>
 * Currently the following two dithering algorithm's are supported :
 * <ol>
 * <li>Simple reduction by reducing the bit width of the color components
 * <li>Reduction by reducing the bit width and doing some rudimentary error
 *     correction on neighbouring pixels.
 * </ol>
 *
 * @see <a href="doc-files/DitherImplementation.html">Implementation of Image
 * 	Dithering/Color Reduction</a>
 *
 * @version $Revision$, $Date$
 * @author fmeschbe, based on CQ2's rgba2/rgbacolor.c color reduction
 * @since coati
 * @audience wad
 */
public class DitherOp extends AbstractBufferedImageOp {

    /**
     * Indicate the use of the simple bit width color reduction algorithm
     * with no further dithering effects.
     */
    public static final DitherAlgorithm DITHER_NONE =
	new DitherSimple();

    /**
     * Indicate the use of the simple bit width color reduction algorithm
     * plus using the simple error correction algorithm.
     */
    public static final DitherAlgorithm DITHER_SIMPLE_ERROR_CORRECTION  =
	new DitherErrorCorrection();

    /** The maximum number of colors to accept after reduction. */
    private final int nCol;

    /**
     * The Color to use for transparency. It is guaranteed, that this color is
     * part of the colors remaining in the image.
     */
    private final Color transparency;

    /**
     * The Color to use for the background. It is guaranteed, that this color is
     * part of the colors remaining in the image.
     */
    private final Color bgcolor;

    /** The algorithm used for color reduction. */
    private final DitherAlgorithm algorithm;

    /**
     * Creates a new <code>DitherOp</code> instance to reduce colors of an
     * image to the given number of colors.
     *
     * @param nCol The maximum number of colors to reduce the image to. This
     * 		must be higher than 2.
     * @param transparency The <code>Color</code> of the color to be considered
     * 		as transparent. This will be one of the nCol colors if not
     * 		<code>null</code>.
     * @param bgcolor The <code>Color</code> of the color to be considered
     * 		as the background. This will be one of the nCol colors if
     * 		not <code>null</code>.
     * @param algorithm The dithering algorithm to be used.
     * @param hints The RenderingHints for the filter operation. This parameter
     * 		may be <code>null</code> and is not currently used.
     *
     * @throws NullPointerException if algorithm is <code>null</code>.
     * @throws IllegalArgumentException if nCol is less than 2.
     */
    public DitherOp(int nCol, Color transparency, Color bgcolor,
	DitherAlgorithm algorithm, RenderingHints hints) {

	// init base class
	super(hints);

	// check algorithm
	if (algorithm == null) {
	    throw new NullPointerException("algorithm");
	}

	// Check nCol
	if (nCol < 2) {
	    throw new IllegalArgumentException("nCol < 2");
	}

	this.nCol = nCol;
	this.transparency = transparency;
	this.bgcolor = bgcolor;
	this.algorithm = algorithm;
    }

    /**
     * Converts the source image to an image with IndexColorModel and with
     * a maximal number of colors. If the source image already has an IndexColorModel and
     * the map size of the that color model instance
     * (<code>src.getColorModel().getMapSize()</code>) is less than or equal to
     * the number of colors desired, the source image is returned. Else a new
     * image is returned.
     *
     * @param src The source image to convert to the <code>IndexColorModel</code>.
     * @param nCol The maximum number of colors to reduce the image to. This
     * 		must be higher than 2.
     * @param transparency The <code>Color</code> of the color to be considered
     * 		as transparent. This will be one of the nCol colors if not
     * 		<code>null</code>.
     * @param bgcolor The <code>Color</code> of the color to be considered
     * 		as the background. This will be one of the nCol colors if
     * 		not <code>null</code>.
     * @param hints The RenderingHints for the filter operation. This parameter
     * 		may be <code>null</code> and is not currently used.
     *
     * @return An image with the <code>IndexColorModel</code> color model. If
     * 		the source image already has the <code>IndexColorModel</code>
     * 		and the number of colors are less than or equal to
     * 		<code>nCol</code> the source image is returned else a new
     * 		image according to the parameters is returned.
     *
     * @throws NullPointerException if the source image is <code>null</code>.
     * @throws NullPointerException if algorithm is <code>null</code>.
     * @throws IllegalArgumentException if nCol is less than 2.
     */
    public static BufferedImage convertToIndexed(BufferedImage src, int nCol,
	Color transparency, Color bgcolor, RenderingHints hints) {
		return null;
	}

    //---------- BufferedImageOp interface -------------------------------------

    /**
     * Performs the operation on a BufferedImage. This implementation only cares
     * to make the images compatible and calls the
     * {@link #doFilter(BufferedImage, BufferedImage)} to do the actual
     * filtering operation.
     * <p>
     * If the color models for the two images do not match, a color
     * conversion into the destination color model will be performed.
     * If the destination image is null,
     * a BufferedImage with an appropriate ColorModel will be created.
     * <p>
     * Note: The dest image might be clipped if it is not big enough to take
     * the complete resized image.
     * <p>
     * This method is overwritten to make sure the pixel data is premultiplied
     * with the alpha value to take the real alpha value into account.
     *
     * @param src The src image to be resized.
     * @param dst The dest image into which to place the resized image. This
     * 		may be <code>null</code> in which case a new image with the
     * 		correct size will be created.
     *
     * @return The newly created image (if dest was <code>null</code>) or dest
     * 		into which the resized src image has been drawn.
     *
     * @throws IllegalArgumentException if the dest image is the same as the
     * 		src image.
     * @throws NullPointerException if the src image is <code>null</code>.
     */
    public BufferedImage filter(BufferedImage src, BufferedImage dst) {
		return null;
	}
    //---------- protected -----------------------------------------------------

    /**
     * Analize and reduce the number of colors of the given image. If the number
     * of colors in the image is less than required number, no image
     * manipulation is done and the source image is drawn into the destination
     * image.
     *
     * @param srcIm The <code>BufferedImage</code> to submit to color reduction.
     * @param dstIm The <code>BufferedImage</code> to use for the image with
     * 		at most the number of colors defined with the constructor.
     *
     * @throws NullPointerException if either srcIm or dstIm is <code>null</code>.
     */
    protected void doFilter(BufferedImage srcIm, BufferedImage dstIm) {
	}

    //---------- internal helper -----------------------------------------------

    private void prepareSourceImage(BufferedImage srcIm) {
	}

    private OctTree buildColorTree(Raster srcRas) {
		return null;
	}

    private byte[] indexColorTree(OctTree tree, boolean createCMap) {
		return null;
	}

    //---------- dither algorithms ---------------------------------------------

    public static interface DitherAlgorithm {
	public void dither(Raster src, WritableRaster dst, OctTree tree) {
	}
    }

    private static class DitherSimple implements DitherAlgorithm {
	public void dither(Raster src, WritableRaster dst, OctTree tree) {
	}
    }

    private static class DitherErrorCorrection implements DitherAlgorithm {
	public void dither(Raster src, WritableRaster dst, OctTree tree) {
	}
    }

    //---------- internal classes ----------------------------------------------

    /**
     * The <code>OctTreeNode</code> class represents one node in the
     * OctTree.
     */
    private static final class OctTreeNode {

	/** The weighted red component */
	public int sr;

	/** The weighted green component */
	public int sg;

	/** The weighted blue component */
	public int sb;

	/** The unweighted red component, set by {@link #indexTree()}. */
	public int r;

	/** The unweighted green component, set by {@link #indexTree()}. */
	public int g;

	/** The unweighted blue component, set by {@link #indexTree()}. */
	public int b;

	/** The alpha value, opaque by default. */
	public int a = 255;

        /**
         * The list of children of this node. If this node is a leaf, there
         * are no children.
         */
        public OctTreeNode[] childs = new OctTreeNode[8];

        /**
         * The parent of this node. This is only used for tree reduction.
         */
        public OctTreeNode parent;

        /** Set to <code>true</code> if this node is a leaf node. */
        public boolean leaf;

        /**
         * The number of pixels bearing the color value represented by this
         * node, if this is a leaf node. Else this is not valid and/or set.
	 * <p>
	 * When building the color map this field is used to store the index
	 * number of the color in.
         */
        public int nofcolors;

        /** The level of the insertion point of this node. */
        public int depth;

	/**
	 * The next node in the color reduction lists ({@link OctTree#redtable}.
	 */
	public OctTreeNode next;

        /**
         * Create a node under the given parent. The parent is linked and the
         * depth is set based on the depth of the parent. If the parent is null
         * the depth set is 0 else it is one higher than the depth of the
         * parent.
         *
         * @param parent The parent under which this node will be located or
         *        null if this is a root or anchor node.
         */
        private OctTreeNode(OctTreeNode parent) {
            this.depth = (parent != null) ? parent.depth + 1 : 0;
            this.parent = parent;
            this.leaf = false;            // Of course we are not a leaf
	    this.next = null;
        }

        /**
         * Recurse down the tree, visiting all leaf nodes and calculating their
	 * real unweighted rgb color values.
         * <p>
         * This method is not intended to be used by the OctTree clients
	 * directly but is called by the {@link OctTree#indexTree} method.
         */
        public void indexTree() {
		}

    }

    /**
     * The <code>OctTree</code> class provides the data structure to build
     * a tree of color values which is very easily traversed and simplifies
     * color reduction on a bit by bit basis.
     * <p>
     * NOTE: This class implements a specialization of the general octal tree.
     * <p>
     * Initially - before reduction - the tree has a fixed height of eight
     * where each node has 0 (only leafs) to 8 children and leafs are only
     * located at the bottom.
     * <p>
     * To reduce a node, the children, which must be leafs themselves, are
     * removed and the node reduced, is changed to a leaf node, effectively
     * reducing the height of the tree. To help identification of nodes to
     * reduce, each node carries a weight value, which in our case of layer
     * colors is equal to the number of pixels having the color identified
     * by the node.
     */
    private static class OctTree {

        /**
         * Maximum depth of the tree. Each node of this depth level
         * is automatically created as a leaf node.
         */
        private static final int MAX_DEPTH = 8;

        /**
	 * How many leafs do we have in the tree. This is the number of
	 * distinct colors managed in the tree.
	 */
        private int leafCount;

        /**
         * Keep a reference to the root node of the tree. This root node
         * is the start for all operations.
         */
        private final OctTreeNode root = new OctTreeNode(null);

        /**
         * Binary trees of all the leaf nodes at specific levels. Initially -
	 * before reduction - all nodes will be in the list number
	 * {@link #MAX_DEPTH}.
         */
        private final OctTreeNode[] redtable = new OctTreeNode[MAX_DEPTH+1];

        /** Counts of nodes contained in each of the lists. */
        private final int[] rednof = new int[MAX_DEPTH+1];

        /**
         * The maximum depth level still containing leaf nodes. This is
         * intially - before reduction - set to {@link #MAX_DEPTH} in the
	 * {@link #buildColorTree} method.
         */
        private int reddepth;

	public OctTree() {
	    for (int i=0; i<redtable.length; i++) {
		redtable[i] = new OctTreeNode(null);
	    }
	}

        /**
         * Index the colors in the tree filling the given palette with
         * the colors in the tree and setting the index value of each node
         * accordingly. Return the length of the palette used.
         */
        public void indexTree() {
		}

	/**
	 * Returns a node, which most closely matches the given rgb color values.
	 * Due to color reduction, this need not be a node for the exact same
	 * color.
	 *
	 * @param r The red color component
	 * @param g The green color component
	 * @param b The blue color component
	 *
	 * @return The node found most closely matching the rgb values or
	 * 	<code>null</code> if no such color exists, which is highly
	 * 	unlikely under normal circumstances.
	 */
	public OctTreeNode getNode(int r, int g, int b) {
		return null;
	}

	/**
         * Insert or modify the leaf node for the given color. If a leaf for
         * the color does not exist yet, it will be created as are any missing
         * parent nodes. If it is existing, the nodes color is weighted by
         * the color.
         *
         * @param r The red color value of the new node
         * @param g The green color value of the new node
         * @param b The blue color value of the new node
         * @param count The number of pixels bearing the given color
         */
        public void insertNode(int r, int g, int b, int count) {
		}

	/**
	 * Returns <code>true</code> if the color tree has been reduced
	 */
	public boolean isReduced() {
		return false;
	}

        /**
         * Identifies a node to reduce and reduce its children. Thus the
         * tree will be reduced and contain upto 7 distinct colors less than
         * before. Nodes reduced away are removed from the tree.
         *
         * @throws NullPointerException if {@link #getReducibleNode} returns
         *    null which is considered a hard error.
         */
        public void reduceNode() {
		}

        //---------- node management -------------------------------------------

	/**
	 * Creates and fills a color map.
	 *
	 * @param cmap The optional byte array to fill with the map. If this is
	 * 		<code>null</code> or not big enough to take all entries
	 * 		a new array is allocated.
	 *
	 * @return The color map either the input cmap or the newly allocated.
	 * 		See comments on cmap above.
	 */
	public byte[] createColorMap(byte[] cmap) {
		return null;
	}

	/**
	 * Deletes the node from the tree defined by its root. If the tree is
	 * empty (root is <code>null</code>) or if the node is not found in the
	 * tree, the 'tree' remains unchanged.
	 *
	 * @param root The root of the tree to delete the node from.
	 * @param node The node to delete from the tree.
	 */
	private void deleteNode(OctTreeNode root, OctTreeNode node) {
	}

	/**
	 * Inserts the node into the binary reduction tree. If the tree is still
	 * empty, that is <code>root</code> is <code>null</code>, the node
	 * will become the root of the tree.
	 *
	 * @param root The root node of the binary tree or <code>null</code> to
	 * 	build a new tree with the node as the root.
	 * @param node The node to insert into the tree or to make the root node
	 * 	of a new tree.
	 */
	private void insertNode(OctTreeNode root, OctTreeNode node) {
	}

        /**
	 * Returns the parent of a reducible node. A node is reducible if the
	 * number of occurrences of the color, the node represents is less than
	 * or equal to the occurrences of all the colors in the same tree.
         *
         * @return The parent of the node with the least number of pixels or
         *      null if no such nodes exist. This latter case is considered
         *      a really hard error and MUST not occurr.
	 *
	 * @throws NullPointerException if the root parameter is
	 * 	<code>null</code>.
         */
        private OctTreeNode getReducibleNode() {
			return null;
		}

    }

}