/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

package org.apache.jackrabbit.oak.plugins.index.lucene;

import java.io.Closeable;
import java.io.File;
import java.io.IOException;
import java.util.Collections;
import java.util.Map;
import java.util.Set;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.ConcurrentMap;
import java.util.concurrent.Executor;
import java.util.concurrent.atomic.AtomicInteger;
import java.util.concurrent.atomic.AtomicLong;

import javax.management.openmbean.CompositeDataSupport;
import javax.management.openmbean.CompositeType;
import javax.management.openmbean.OpenDataException;
import javax.management.openmbean.OpenType;
import javax.management.openmbean.SimpleType;
import javax.management.openmbean.TabularData;
import javax.management.openmbean.TabularDataSupport;
import javax.management.openmbean.TabularType;

import com.google.common.base.Function;
import com.google.common.collect.ImmutableSet;
import com.google.common.collect.Sets;
import org.apache.commons.io.FileUtils;
import org.apache.jackrabbit.oak.plugins.index.lucene.directory.CopyOnReadDirectory;
import org.apache.jackrabbit.oak.plugins.index.lucene.directory.CopyOnWriteDirectory;
import org.apache.jackrabbit.oak.plugins.index.lucene.directory.IndexSanityChecker;
import org.apache.jackrabbit.oak.plugins.index.lucene.directory.DirectoryUtils;
import org.apache.jackrabbit.oak.plugins.index.lucene.directory.IndexRootDirectory;
import org.apache.jackrabbit.oak.plugins.index.lucene.directory.LocalIndexDir;
import org.apache.jackrabbit.oak.plugins.index.lucene.directory.LocalIndexFile;
import org.apache.lucene.store.Directory;
import org.apache.lucene.store.FSDirectory;
import org.apache.lucene.store.FilterDirectory;
import org.apache.lucene.store.NoLockFactory;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import static com.google.common.base.Preconditions.checkState;
import static com.google.common.collect.Iterables.toArray;
import static com.google.common.collect.Iterables.transform;
import static com.google.common.collect.Maps.newConcurrentMap;
import static com.google.common.collect.Maps.newHashMap;
import static org.apache.jackrabbit.oak.commons.IOUtils.humanReadableByteCount;

public class IndexCopier implements CopyOnReadStatsMBean, Closeable {
    public static final Set<String> REMOTE_ONLY = ImmutableSet.of("segments.gen");
    private static final int MAX_FAILURE_ENTRIES = 10000;
    private static final String WORK_DIR_NAME = "indexWriterDir";

    private final Logger log = LoggerFactory.getLogger(getClass());
    private final Executor executor;
    private final File indexWorkDir;

    private final AtomicInteger readerLocalReadCount = new AtomicInteger();
    private final AtomicInteger writerLocalReadCount = new AtomicInteger();
    private final AtomicInteger readerRemoteReadCount = new AtomicInteger();
    private final AtomicInteger writerRemoteReadCount = new AtomicInteger();
    private final AtomicInteger invalidFileCount = new AtomicInteger();
    private final AtomicInteger deletedFileCount = new AtomicInteger();
    private final AtomicInteger scheduledForCopyCount = new AtomicInteger();
    private final AtomicInteger copyInProgressCount = new AtomicInteger();
    private final AtomicInteger maxCopyInProgressCount = new AtomicInteger();
    private final AtomicInteger maxScheduledForCopyCount = new AtomicInteger();
    private final AtomicInteger uploadCount = new AtomicInteger();
    private final AtomicInteger downloadCount = new AtomicInteger();
    private final AtomicLong copyInProgressSize = new AtomicLong();
    private final AtomicLong downloadSize = new AtomicLong();
    private final AtomicLong uploadSize = new AtomicLong();
    private final AtomicLong garbageCollectedSize = new AtomicLong();
    private final AtomicLong skippedFromUploadSize = new AtomicLong();
    private final AtomicLong downloadTime = new AtomicLong();
    private final AtomicLong uploadTime = new AtomicLong();


    private final Map<String, Set<String>> sharedWorkingSetMap = newHashMap();
    private final Map<String, String> indexPathVersionMapping = newConcurrentMap();
    private final ConcurrentMap<String, LocalIndexFile> failedToDeleteFiles = newConcurrentMap();
    private final Set<LocalIndexFile> copyInProgressFiles = Collections.newSetFromMap(new ConcurrentHashMap<LocalIndexFile, Boolean>());
    private final boolean prefetchEnabled;
    private volatile boolean closed;
    private final IndexRootDirectory indexRootDirectory;
    private final Set<String> validatedIndexPaths = Sets.newConcurrentHashSet();

    public IndexCopier(Executor executor, File indexRootDir) throws IOException {
        this(executor, indexRootDir, false);
    }

    public IndexCopier(Executor executor, File indexRootDir, boolean prefetchEnabled) throws IOException {
        this.executor = executor;
        this.prefetchEnabled = prefetchEnabled;
        this.indexWorkDir = initializerWorkDir(indexRootDir);
        this.indexRootDirectory = new IndexRootDirectory(indexRootDir);
    }

    public Directory wrapForRead(String indexPath, IndexDefinition definition,
                                 Directory remote, String dirName) throws IOException {
									return null;
								}

    public Directory wrapForWrite(IndexDefinition definition, Directory remote, boolean reindexMode, String dirName) throws IOException {
		return null;
	}

    @Override
    public void close() throws IOException {
	}

    public boolean isClosed() {
		return false;
	}

    File getIndexWorkDir() {
		return null;
	}

    IndexRootDirectory getIndexRootDirectory() {
		return null;
	}

    protected Directory createLocalDirForIndexWriter(IndexDefinition definition, String dirName) throws IOException {
		return null;
	}

    protected Directory createLocalDirForIndexReader(String indexPath, IndexDefinition definition, String dirName) throws IOException {
		return null;
	}

    public File getIndexDir(IndexDefinition definition, String indexPath, String dirName) throws IOException {
		return null;
	}

    public void addIndexFileBeingWritten(String indexPath, String name) {
	}

    public void clearIndexFilesBeingWritten(String indexPath) {
	}

    public Set<String> getIndexFilesBeingWritten(String indexPath) {
		return null;
	}

    Map<String, LocalIndexFile> getFailedToDeleteFiles() {
		return null;
	}

    private void failedToDelete(LocalIndexFile file){
	}

    private void successfullyDeleted(LocalIndexFile file, boolean fileExisted){
	}

    /**
     * Provide the corresponding shared state to enable COW inform COR
     * about new files it is creating while indexing. This would allow COR to ignore
     * such files while determining the deletion candidates.
     *
     * @param defn index definition for which the directory is being created
     * @return a set to maintain the state of new files being created by the COW Directory
     */
    private Set<String> getSharedWorkingSet(String indexPath){
		return null;
	}

    private void checkIntegrity(String indexPath, Directory local, Directory remote) throws IOException {
	}

    /**
     * Creates the workDir. If it exists then it is cleaned
     *
     * @param indexRootDir root directory under which all indexing related files are managed
     * @return work directory. Always empty
     */
    private static File initializerWorkDir(File indexRootDir) throws IOException {
		return null;
	}

    /**
     * Create a unique key based on indexPath and dirName used under that path
     */
    private static String createIndexPathKey(String indexPath, String dirName){
		return null;
	}

    public boolean deleteFile(Directory dir, String fileName, boolean copiedFromRemote){
		return false;
	}

    public long startCopy(LocalIndexFile file) {
		return 0L;
	}

    public boolean isCopyInProgress(LocalIndexFile file) {
		return false;
	}

    public void doneCopy(LocalIndexFile file, long start) {
	}

    private void updateMaxScheduled(int val) {
	}

    private void updateMaxInProgress(int val) {
	}


    private class DeleteOldDirOnClose extends FilterDirectory {
        private final File oldIndexDir;

        protected DeleteOldDirOnClose(Directory in, File oldIndexDir) {
            super(in);
            this.oldIndexDir = oldIndexDir;
        }

        @Override
        public void close() throws IOException {
		}

        @Override
        public String toString() {
			return null;
		}
    }

    //~------------------------------------------< Stats Collection >

    public void skippedUpload(long skippedFilesSize) {
	}

    public void scheduledForCopy() {
	}

    public void copyDone(){
	}

    public void readFromRemote(boolean reader) {
	}

    public void readFromLocal(boolean reader) {
	}

    public void foundInvalidFile(){
	}

    //~------------------------------------------< CopyOnReadStatsMBean >

    @Override
    public TabularData getIndexPathMapping() {
		return null;
	}

    @Override
    public boolean isPrefetchEnabled() {
		return false;
	}

    @Override
    public int getReaderLocalReadCount() {
		return 0;
	}

    @Override
    public int getReaderRemoteReadCount() {
		return 0;
	}

    @Override
    public int getWriterLocalReadCount() {
		return 0;
	}

    @Override
    public int getWriterRemoteReadCount() {
		return 0;
	}

    public int getInvalidFileCount(){
		return 0;
	}

    @Override
    public String getDownloadSize() {
		return null;
	}

    @Override
    public long getDownloadTime() {
		return 0L;
	}

    @Override
    public int getDownloadCount() {
		return 0;
	}

    @Override
    public int getUploadCount() {
		return 0;
	}

    @Override
    public String getUploadSize() {
		return null;
	}

    @Override
    public long getUploadTime() {
		return 0L;
	}

    @Override
    public String getLocalIndexSize() {
		return null;
	}

    @Override
    public String[] getGarbageDetails() {
		return null;
	}

    @Override
    public String getGarbageSize() {
		return null;
	}

    @Override
    public int getScheduledForCopyCount() {
		return 0;
	}

    @Override
    public int getCopyInProgressCount() {
		return 0;
	}

    @Override
    public String getCopyInProgressSize() {
		return null;
	}

    @Override
    public int getMaxCopyInProgressCount() {
		return 0;
	}

    @Override
    public int getMaxScheduledForCopyCount() {
		return 0;
	}

    public String getSkippedFromUploadSize() {
		return null;
	}

    @Override
    public String[] getCopyInProgressDetails() {
		return null;
	}

    @Override
    public int getDeletedFilesCount() {
		return 0;
	}

    @Override
    public String getGarbageCollectedSize() {
		return null;
	}

    private static class IndexMappingData {
        static final String[] FIELD_NAMES = new String[]{
                "jcrPath",
                "fsPath",
                "size",
        };

        static final String[] FIELD_DESCRIPTIONS = new String[]{
                "JCR Path",
                "Filesystem Path",
                "Size",
        };

        static final OpenType[] FIELD_TYPES = new OpenType[]{
                SimpleType.STRING,
                SimpleType.STRING,
                SimpleType.STRING,
        };

        static final CompositeType TYPE = createCompositeType();

        static CompositeType createCompositeType() {
			return null;
		}
    }
}
