/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

package org.apache.jackrabbit.oak.plugins.index.lucene;

import java.io.InputStream;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.Locale;
import java.util.Map;
import java.util.Set;
import java.util.regex.Pattern;

import javax.annotation.CheckForNull;
import javax.annotation.Nonnull;
import javax.annotation.Nullable;
import javax.jcr.PropertyType;
import javax.jcr.RepositoryException;
import javax.jcr.nodetype.NodeTypeIterator;

import com.google.common.collect.ImmutableList;
import com.google.common.collect.ImmutableMap;
import com.google.common.collect.ImmutableSet;
import com.google.common.collect.Iterables;
import com.google.common.collect.Sets;
import com.google.common.primitives.Ints;
import org.apache.jackrabbit.JcrConstants;
import org.apache.jackrabbit.oak.api.PropertyState;
import org.apache.jackrabbit.oak.api.Tree;
import org.apache.jackrabbit.oak.api.Type;
import org.apache.jackrabbit.oak.commons.PathUtils;
import org.apache.jackrabbit.oak.namepath.NamePathMapper;
import org.apache.jackrabbit.oak.plugins.index.IndexConstants;
import org.apache.jackrabbit.oak.plugins.index.PathFilter;
import org.apache.jackrabbit.oak.plugins.index.lucene.util.ConfigUtil;
import org.apache.jackrabbit.oak.plugins.index.lucene.util.FunctionIndexProcessor;
import org.apache.jackrabbit.oak.plugins.index.lucene.util.TokenizerChain;
import org.apache.jackrabbit.oak.plugins.memory.PropertyStates;
import org.apache.jackrabbit.oak.plugins.nodetype.ReadOnlyNodeTypeManager;
import org.apache.jackrabbit.oak.plugins.tree.TreeFactory;
import org.apache.jackrabbit.oak.spi.query.QueryIndex.OrderEntry;
import org.apache.jackrabbit.oak.spi.state.ChildNodeEntry;
import org.apache.jackrabbit.oak.spi.state.NodeBuilder;
import org.apache.jackrabbit.oak.spi.state.NodeState;
import org.apache.jackrabbit.oak.spi.state.NodeStateUtils;
import org.apache.jackrabbit.oak.spi.state.ReadOnlyBuilder;
import org.apache.jackrabbit.oak.util.TreeUtil;
import org.apache.lucene.analysis.Analyzer;
import org.apache.lucene.analysis.miscellaneous.LimitTokenCountAnalyzer;
import org.apache.lucene.analysis.miscellaneous.PerFieldAnalyzerWrapper;
import org.apache.lucene.analysis.path.PathHierarchyTokenizerFactory;
import org.apache.lucene.codecs.Codec;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import static com.google.common.base.Preconditions.checkNotNull;
import static com.google.common.collect.Lists.newArrayList;
import static com.google.common.collect.Lists.newArrayListWithCapacity;
import static com.google.common.collect.Maps.newHashMap;
import static com.google.common.collect.Sets.newHashSet;
import static org.apache.jackrabbit.JcrConstants.JCR_SCORE;
import static org.apache.jackrabbit.JcrConstants.JCR_SYSTEM;
import static org.apache.jackrabbit.JcrConstants.NT_BASE;
import static org.apache.jackrabbit.oak.api.Type.NAMES;
import static org.apache.jackrabbit.oak.commons.PathUtils.getParentPath;
import static org.apache.jackrabbit.oak.plugins.index.IndexConstants.DECLARING_NODE_TYPES;
import static org.apache.jackrabbit.oak.plugins.index.IndexConstants.ENTRY_COUNT_PROPERTY_NAME;
import static org.apache.jackrabbit.oak.plugins.index.IndexConstants.INDEXING_MODE_NRT;
import static org.apache.jackrabbit.oak.plugins.index.IndexConstants.INDEXING_MODE_SYNC;
import static org.apache.jackrabbit.oak.plugins.index.IndexConstants.REINDEX_COUNT;
import static org.apache.jackrabbit.oak.plugins.index.lucene.LuceneIndexConstants.*;
import static org.apache.jackrabbit.oak.plugins.index.lucene.PropertyDefinition.DEFAULT_BOOST;
import static org.apache.jackrabbit.oak.plugins.index.lucene.util.ConfigUtil.getOptionalValue;
import static org.apache.jackrabbit.oak.plugins.memory.EmptyNodeState.EMPTY_NODE;
import static org.apache.jackrabbit.oak.plugins.nodetype.NodeTypeConstants.JCR_NODE_TYPES;
import static org.apache.jackrabbit.oak.plugins.nodetype.NodeTypeConstants.NODE_TYPES_PATH;

public final class IndexDefinition implements Aggregate.AggregateMapper {
    /**
     * Name of the internal property that contains the child order defined in
     * org.apache.jackrabbit.oak.plugins.tree.impl.TreeConstants
     */
    private static final String OAK_CHILD_ORDER = ":childOrder";

    private static final Logger log = LoggerFactory.getLogger(IndexDefinition.class);

    private static boolean disableStoredIndexDefinition;

    /**
     * Default number of seconds after which to delete actively. Default is -1, meaning disabled.
     * The plan is to use 3600 (1 hour) in the future.
     */
    static final int DEFAULT_ACTIVE_DELETE = -1; // 60 * 60;

    /**
     * Blob size to use by default. To avoid issues in OAK-2105 the size should not
     * be power of 2.
     */
    static final int DEFAULT_BLOB_SIZE = 1024 * 1024 - 1024;

    /**
     * Default entry count to keep estimated entry count low.
     */
    static final long DEFAULT_ENTRY_COUNT = 1000;

    /**
     * Default value for property {@link #maxFieldLength}.
     */
    public static final int DEFAULT_MAX_FIELD_LENGTH = 10000;

    static final int DEFAULT_MAX_EXTRACT_LENGTH = -10;

    /**
     * System managed hidden property to record the current index version
     */
    static final String INDEX_VERSION = ":version";

    /**
     * Hidden node under index definition which is used to store the index definition
     * nodestate as it was at time of reindexing
     */
    static final String INDEX_DEFINITION_NODE = ":index-definition";

    /**
     * Hidden node under index definition which is used to store meta info
     */
    static final String STATUS_NODE = ":status";

    /**
     * Meta property which provides the unique id
     */
    static final String PROP_UID = "uid";

    private static String TYPES_ALLOW_ALL_NAME = "all";

    static final int TYPES_ALLOW_NONE = PropertyType.UNDEFINED;

    static final int TYPES_ALLOW_ALL = -1;

    /**
     * Default suggesterUpdateFrequencyMinutes
     */
    static final int DEFAULT_SUGGESTER_UPDATE_FREQUENCY_MINUTES = 10;

    /**
     * Default no. of facets retrieved
     */
    static final int DEFAULT_FACET_COUNT = 10;

    /**
     * native sort order
     */
    static final OrderEntry NATIVE_SORT_ORDER = new OrderEntry(JCR_SCORE, Type.UNDEFINED,
        OrderEntry.Order.DESCENDING);

    private final boolean fullTextEnabled;

    private final NodeState definition;

    private final NodeState root;

    private final String funcName;

    private final int activeDelete;
    
    private final int blobSize;

    private final Codec codec;

    /**
     * Defines the maximum estimated entry count configured.
     * Defaults to {#DEFAULT_ENTRY_COUNT}
     */
    private final long entryCount;

    private final boolean entryCountDefined;

    private final double costPerEntry;

    private final double costPerExecution;

    /**
     * The {@link IndexingRule}s inside this configuration. Keys being the NodeType names
     */
    private final Map<String, List<IndexingRule>> indexRules;

    private final List<IndexingRule> definedRules;

    private final String indexName;

    private final boolean testMode;

    private final boolean evaluatePathRestrictions;

    private final IndexFormatVersion version;

    private final Map<String, Aggregate> aggregates;

    private final boolean indexesAllTypes;

    private final Analyzer analyzer;

    private final String scorerProviderName;

    private final Map<String, Analyzer> analyzers;

    private final boolean hasCustomTikaConfig;

    private final int maxFieldLength;

    private final int maxExtractLength;

    private final int suggesterUpdateFrequencyMinutes;

    private final long reindexCount;

    private final PathFilter pathFilter;

    @Nullable
    private final String[] queryPaths;

    private final boolean saveDirListing;

    private final boolean suggestAnalyzed;

    private final boolean secureFacets;

    private final int numberOfTopFacets;

    private final boolean suggestEnabled;

    private final boolean spellcheckEnabled;

    private final String indexPath;

    private final boolean nrtIndexMode;
    private final boolean syncIndexMode;

    @Nullable
    private final String uid;

    //~--------------------------------------------------------< Builder >

    public static Builder newBuilder(NodeState root, NodeState defn, String indexPath){
		return null;
	}

    public static class Builder {
        /**
         * Default unique id used when no existing uid is defined
         * and index is not populated
         */
        private static final String DEFAULT_UID = "0";
        private final NodeState root;
        private final NodeState defn;
        private final String indexPath;
        private IndexFormatVersion version;
        private String uid;
        private boolean reindexMode;

        public Builder(NodeState root, NodeState defn, String indexPath) {
            this.root = checkNotNull(root);
            this.defn = checkNotNull(defn);
            this.indexPath = checkNotNull(indexPath);
        }

        public Builder version(IndexFormatVersion version){
			return null;
		}

        public Builder uid(String uid){
			return null;
		}

        public Builder reindex(){
			return null;
		}

        public IndexDefinition build(){
			return null;
		}
    }

    public IndexDefinition(NodeState root, NodeState defn, String indexPath) {
        this(root, getIndexDefinitionState(defn), determineIndexFormatVersion(defn), determineUniqueId(defn), indexPath);
    }

    private IndexDefinition(NodeState root, NodeState defn, IndexFormatVersion version, String uid, String indexPath) {
        this.root = root;
        this.version = checkNotNull(version);
        this.uid = uid;
        this.definition = defn;
        this.indexPath = checkNotNull(indexPath);
        this.indexName = indexPath;

        this.blobSize = getOptionalValue(defn, BLOB_SIZE, DEFAULT_BLOB_SIZE);
        this.activeDelete = getOptionalValue(defn, ACTIVE_DELETE, DEFAULT_ACTIVE_DELETE);
        this.testMode = getOptionalValue(defn, LuceneIndexConstants.TEST_MODE, false);

        this.aggregates = collectAggregates(defn);

        NodeState rulesState = defn.getChildNode(LuceneIndexConstants.INDEX_RULES);
        if (!rulesState.exists()){
            rulesState = createIndexRules(defn).getNodeState();
        }

        List<IndexingRule> definedIndexRules = newArrayList();
        this.indexRules = collectIndexRules(rulesState, definedIndexRules);
        this.definedRules = ImmutableList.copyOf(definedIndexRules);

        this.fullTextEnabled = hasFulltextEnabledIndexRule(definedIndexRules);
        this.evaluatePathRestrictions = getOptionalValue(defn, EVALUATE_PATH_RESTRICTION, false);

        String functionName = getOptionalValue(defn, LuceneIndexConstants.FUNC_NAME, null);
        if (fullTextEnabled && functionName == null){
            functionName = "lucene";
        }
        this.funcName = functionName != null ? "native*" + functionName : null;

        this.codec = createCodec();

        if (defn.hasProperty(ENTRY_COUNT_PROPERTY_NAME)) {
            this.entryCountDefined = true;
            this.entryCount = defn.getProperty(ENTRY_COUNT_PROPERTY_NAME).getValue(Type.LONG);
        } else {
            this.entryCountDefined = false;
            this.entryCount = DEFAULT_ENTRY_COUNT;
        }

        this.maxFieldLength = getOptionalValue(defn, LuceneIndexConstants.MAX_FIELD_LENGTH, DEFAULT_MAX_FIELD_LENGTH);
        this.costPerEntry = getOptionalValue(defn, LuceneIndexConstants.COST_PER_ENTRY, getDefaultCostPerEntry(version));
        this.costPerExecution = getOptionalValue(defn, LuceneIndexConstants.COST_PER_EXECUTION, 1.0);
        this.indexesAllTypes = areAllTypesIndexed();
        this.analyzers = collectAnalyzers(defn);
        this.analyzer = createAnalyzer();
        this.hasCustomTikaConfig = getTikaConfigNode().exists();
        this.maxExtractLength = determineMaxExtractLength();
        this.suggesterUpdateFrequencyMinutes = evaluateSuggesterUpdateFrequencyMinutes(defn,
                DEFAULT_SUGGESTER_UPDATE_FREQUENCY_MINUTES);
        this.scorerProviderName = getOptionalValue(defn, LuceneIndexConstants.PROP_SCORER_PROVIDER, null);
        this.reindexCount = getOptionalValue(defn, REINDEX_COUNT, 0);
        this.pathFilter = PathFilter.from(new ReadOnlyBuilder(defn));
        this.queryPaths = getQueryPaths(defn);
        this.saveDirListing = getOptionalValue(defn, LuceneIndexConstants.SAVE_DIR_LISTING, true);
        this.suggestAnalyzed = evaluateSuggestAnalyzed(defn, false);

        if (defn.hasChildNode(FACETS)) {
            NodeState facetsConfig =  defn.getChildNode(FACETS);
            this.secureFacets = getOptionalValue(facetsConfig, PROP_SECURE_FACETS, true);
            this.numberOfTopFacets = getOptionalValue(facetsConfig, PROP_FACETS_TOP_CHILDREN, DEFAULT_FACET_COUNT);
        } else {
            this.secureFacets = true;
            this.numberOfTopFacets = DEFAULT_FACET_COUNT;
        }

        this.suggestEnabled = evaluateSuggestionEnabled();
        this.spellcheckEnabled = evaluateSpellcheckEnabled();
        this.nrtIndexMode = supportsNRTIndexing(defn);
        this.syncIndexMode = supportsSyncIndexing(defn);
    }

    public NodeState getDefinitionNodeState() {
		return null;
	}

    public boolean isFullTextEnabled() {
		return false;
	}

    public String getFunctionName(){
		return null;
	}

    public boolean hasFunctionDefined(){
		return false;
	}

    /**
     * Size in bytes for the blobs created while storing the index content
     * @return size in bytes
     */
    public int getBlobSize() {
		return 0;
	}

    @CheckForNull
    public Codec getCodec() {
		return null;
	}

    public long getReindexCount(){
		return 0L;
	}

    public long getEntryCount() {
		return 0L;
	}

    private int evaluateSuggesterUpdateFrequencyMinutes(NodeState defn, int defaultValue) {
		return 0;
	}

    public int getSuggesterUpdateFrequencyMinutes() {
		return 0;
	}

    public boolean isEntryCountDefined() {
		return false;
	}

    public double getCostPerEntry() {
		return 0.0d;
	}

    public double getCostPerExecution() {
		return 0.0d;
	}

    public long getFulltextEntryCount(long numOfDocs){
		return 0L;
	}

    public IndexFormatVersion getVersion() {
		return null;
	}

    public boolean isOfOldFormat(){
		return false;
	}

    public boolean isTestMode() {
		return false;
	}

    public boolean evaluatePathRestrictions() {
		return false;
	}

    public Analyzer getAnalyzer(){
		return null;
	}

    public boolean hasCustomTikaConfig(){
		return false;
	}

    public InputStream getTikaConfig(){
		return null;
	}

    public String getIndexName() {
		return null;
	}

    public int getMaxExtractLength() {
		return 0;
	}

    public String getScorerProviderName() {
		return null;
	}

    public boolean saveDirListing() {
		return false;
	}

    public PathFilter getPathFilter() {
		return null;
	}

    @Nullable
    public String[] getQueryPaths() {
		return null;
	}

    @CheckForNull
    public String getUniqueId() {
		return null;
	}

    public boolean isNRTIndexingEnabled() {
		return false;
	}

    public boolean isSyncIndexingEnabled() {
		return false;
	}

    /**
     * Check if the index definition is fresh of some index has happened
     *
     * @param definition nodestate for Index Definition
     * @return true if index has some indexed content
     */
    public static boolean hasPersistedIndex(NodeState definition){
		return false;
	}

    public static boolean isDisableStoredIndexDefinition() {
		return false;
	}

    public static void setDisableStoredIndexDefinition(boolean disableStoredIndexDefinitionDefault) {
	}

    @Override
    public String toString() {
		return null;
	}

    //~---------------------------------------------------< Analyzer >

    private Analyzer createAnalyzer() {
		return null;
	}

    private static Map<String, Analyzer> collectAnalyzers(NodeState defn) {
		return null;
	}

    //~---------------------------------------------------< Aggregates >

    @CheckForNull
    public Aggregate getAggregate(String nodeType){
		return null;
	}

    private Map<String, Aggregate> collectAggregates(NodeState defn) {
		return null;
	}

    //~---------------------------------------------------< IndexRule >

    public boolean hasMatchingNodeTypeReg(NodeState root){
		return false;
	}


    public List<IndexingRule> getDefinedRules() {
		return null;
	}

    @CheckForNull
    public IndexingRule getApplicableIndexingRule(String primaryNodeType) {
		return null;
	}

    /**
     * Returns the first indexing rule that applies to the given node
     * <code>state</code>.
     *
     * @param state a node state.
     * @return the indexing rule or <code>null</code> if none applies.
     */
    @CheckForNull
    public IndexingRule getApplicableIndexingRule(NodeState state) {
		return null;
	}

    private Map<String, List<IndexingRule>> collectIndexRules(NodeState indexRules,
                                                              List<IndexingRule> definedIndexRules){
																return null;
															}

    private boolean areAllTypesIndexed() {
		return false;
	}

    private boolean evaluateSuggestionEnabled() {
		return false;
	}

    public boolean isSuggestEnabled() {
		return false;
	}

    private boolean evaluateSpellcheckEnabled() {
		return false;
	}

    public boolean isSpellcheckEnabled() {
		return false;
	}

    public String getIndexPath() {
		return null;
	}

    private boolean evaluateSuggestAnalyzed(NodeState defn, boolean defaultValue) {
		return false;
	}

    public boolean isSuggestAnalyzed() {
		return false;
	}

    public boolean isSecureFacets() {
		return false;
	}

    public int getNumberOfTopFacets() {
		return 0;
	}

    public class IndexingRule {
        private final String baseNodeType;
        private final String nodeTypeName;
        /**
         * Case insensitive map of lower cased propertyName to propertyConfigs
         */
        private final Map<String, PropertyDefinition> propConfigs;
        private final List<NamePattern> namePatterns;
        private final List<PropertyDefinition> nullCheckEnabledProperties;
        private final List<PropertyDefinition> functionRestrictions;
        private final List<PropertyDefinition> notNullCheckEnabledProperties;
        private final List<PropertyDefinition> nodeScopeAnalyzedProps;
        private final boolean indexesAllNodesOfMatchingType;
        private final boolean nodeNameIndexed;

        final float boost;
        final boolean inherited;
        final int propertyTypes;
        final boolean fulltextEnabled;
        final boolean propertyIndexEnabled;
        final boolean nodeFullTextIndexed;

        final Aggregate aggregate;
        final Aggregate propAggregate;


        IndexingRule(String nodeTypeName, NodeState config) {
            this.nodeTypeName = nodeTypeName;
            this.baseNodeType = nodeTypeName;
            this.boost = getOptionalValue(config, FIELD_BOOST, DEFAULT_BOOST);
            this.inherited = getOptionalValue(config, LuceneIndexConstants.RULE_INHERITED, true);
            this.propertyTypes = getSupportedTypes(config, INCLUDE_PROPERTY_TYPES, TYPES_ALLOW_ALL);

            List<NamePattern> namePatterns = newArrayList();
            List<PropertyDefinition> nonExistentProperties = newArrayList();
            List<PropertyDefinition> functionRestrictions = newArrayList();
            List<PropertyDefinition> existentProperties = newArrayList();
            List<PropertyDefinition> nodeScopeAnalyzedProps = newArrayList();
            List<Aggregate.Include> propIncludes = newArrayList();
            this.propConfigs = collectPropConfigs(config, namePatterns, propIncludes, nonExistentProperties,
                    existentProperties, nodeScopeAnalyzedProps, functionRestrictions);
            this.propAggregate = new Aggregate(nodeTypeName, propIncludes);
            this.aggregate = combine(propAggregate, nodeTypeName);

            this.namePatterns = ImmutableList.copyOf(namePatterns);
            this.nodeScopeAnalyzedProps = ImmutableList.copyOf(nodeScopeAnalyzedProps);
            this.nullCheckEnabledProperties = ImmutableList.copyOf(nonExistentProperties);
            this.functionRestrictions = ImmutableList.copyOf(functionRestrictions);
            this.notNullCheckEnabledProperties = ImmutableList.copyOf(existentProperties);
            this.fulltextEnabled = aggregate.hasNodeAggregates() || hasAnyFullTextEnabledProperty();
            this.nodeFullTextIndexed = aggregate.hasNodeAggregates() || anyNodeScopeIndexedProperty();
            this.propertyIndexEnabled = hasAnyPropertyIndexConfigured();
            this.indexesAllNodesOfMatchingType = areAlMatchingNodeByTypeIndexed();
            this.nodeNameIndexed = evaluateNodeNameIndexed(config);
            validateRuleDefinition();
        }

        /**
         * Creates a new indexing rule base on an existing one, but for a
         * different node type name.
         *
         * @param original the existing rule.
         * @param nodeTypeName the node type name for the rule.
         */
        IndexingRule(IndexingRule original, String nodeTypeName) {
            this.nodeTypeName = nodeTypeName;
            this.baseNodeType = original.getNodeTypeName();
            this.propConfigs = original.propConfigs;
            this.namePatterns = original.namePatterns;
            this.boost = original.boost;
            this.inherited = original.inherited;
            this.propertyTypes = original.propertyTypes;
            this.propertyIndexEnabled = original.propertyIndexEnabled;
            this.propAggregate = original.propAggregate;
            this.nullCheckEnabledProperties = original.nullCheckEnabledProperties;
            this.notNullCheckEnabledProperties = original.notNullCheckEnabledProperties;
            this.functionRestrictions = original.functionRestrictions;
            this.nodeScopeAnalyzedProps = original.nodeScopeAnalyzedProps;
            this.aggregate = combine(propAggregate, nodeTypeName);
            this.fulltextEnabled = aggregate.hasNodeAggregates() || original.fulltextEnabled;
            this.nodeFullTextIndexed = aggregate.hasNodeAggregates() || original.nodeFullTextIndexed;
            this.indexesAllNodesOfMatchingType = areAlMatchingNodeByTypeIndexed();
            this.nodeNameIndexed = original.nodeNameIndexed;
        }

        /**
         * Returns <code>true</code> if the property with the given name is
         * indexed according to this rule.
         *
         * @param propertyName the name of a property.
         * @return <code>true</code> if the property is indexed;
         *         <code>false</code> otherwise.
         */
        public boolean isIndexed(String propertyName) {
			return false;
		}


        /**
         * Returns the name of the node type where this rule applies to.
         *
         * @return name of the node type.
         */
        public String getNodeTypeName() {
			return null;
		}

        /**
         * @return name of the base node type.
         */
        public String getBaseNodeType() {
			return null;
		}

        public List<PropertyDefinition> getNullCheckEnabledProperties() {
			return null;
		}
        
        public List<PropertyDefinition> getFunctionRestrictions() {
			return null;
		}

        public List<PropertyDefinition> getNotNullCheckEnabledProperties() {
			return null;
		}

        public List<PropertyDefinition> getNodeScopeAnalyzedProps() {
			return null;
		}

        @Override
        public String toString() {
			return null;
		}

        public boolean isAggregated(String nodePath) {
			return false;
		}

        /**
         * Returns <code>true</code> if this rule applies to the given node
         * <code>state</code>.
         *
         * @param state the state to check.
         * @return <code>true</code> the rule applies to the given node;
         *         <code>false</code> otherwise.
         */
        public boolean appliesTo(NodeState state) {
			return false;
		}

        public boolean appliesTo(String nodeTypeName) {
			return false;
		}

        public boolean isNodeNameIndexed() {
			return false;
		}

        /**
         * Returns true if the rule has any property definition which enables
         * evaluation of fulltext related clauses
         */
        public boolean isFulltextEnabled() {
			return false;
		}

        /**
         * Returns true if a fulltext field for the node would be created
         * for any node matching the current rule.
         */
        public boolean isNodeFullTextIndexed() {
			return false;
		}

        /**
         * @param propertyName name of a property.
         * @return the property configuration or <code>null</code> if this
         *         indexing rule does not contain a configuration for the given
         *         property.
         */
        @CheckForNull
        public PropertyDefinition getConfig(String propertyName) {
			return null;
		}

        public boolean includePropertyType(int type){
			return false;
		}

        public Aggregate getAggregate() {
			return null;
		}

        /**
         * Flag to determine weather current index rule definition allows indexing of all
         * node of type as covered by the current rule. For example if the rule only indexes
         * certain property 'foo' for node type 'app:Asset' then index would only have
         * entries for those assets where foo is defined. Such an index cannot claim that
         * it has entries for all assets.

         * @return true in case all matching node types are covered by this rule
         */
        public boolean indexesAllNodesOfMatchingType() {
			return false;
		}

        public boolean isBasedOnNtBase(){
			return false;
		}

        private Map<String, PropertyDefinition> collectPropConfigs(NodeState config, 
                                                                   List<NamePattern> patterns,
                                                                   List<Aggregate.Include> propAggregate,
                                                                   List<PropertyDefinition> nonExistentProperties,
                                                                   List<PropertyDefinition> existentProperties,
                                                                   List<PropertyDefinition> nodeScopeAnalyzedProps, 
                                                                   List<PropertyDefinition> functionRestrictions) {
																	return null;
																}

        private boolean hasAnyFullTextEnabledProperty() {
			return false;
		}

        private boolean hasAnyPropertyIndexConfigured() {
			return false;
		}

        private boolean anyNodeScopeIndexedProperty(){
			return false;
		}

        private boolean areAlMatchingNodeByTypeIndexed(){
			return false;
		}

        private boolean evaluateNodeNameIndexed(NodeState config) {
			return false;
		}

        private Aggregate combine(Aggregate propAggregate, String nodeTypeName){
			return null;
		}

        private void validateRuleDefinition() {
		}
    }

    /**
     * A property name pattern.
     */
    private static final class NamePattern {
        private final String parentPath;
        /**
         * The pattern to match.
         */
        private final Pattern pattern;

        /**
         * The associated configuration.
         */
        private final PropertyDefinition config;

        /**
         * Creates a new name pattern.
         *
         * @param pattern the pattern as defined by the property definition
         * @param config the associated configuration.
         */
        private NamePattern(String pattern,
                            PropertyDefinition config){

            //Special handling for all props regex as its already being used
            //and use of '/' in regex would confuse the parent path calculation
            //logic
            if (LuceneIndexConstants.REGEX_ALL_PROPS.equals(pattern)){
                this.parentPath = "";
                this.pattern = Pattern.compile(pattern);
            } else {
                this.parentPath = getParentPath(pattern);
                this.pattern = Pattern.compile(PathUtils.getName(pattern));
            }
            this.config = config;
        }

        /**
         * @param propertyPath property name to match
         * @return <code>true</code> if <code>property name</code> matches this name
         *         pattern; <code>false</code> otherwise.
         */
        boolean matches(String propertyPath) {
			return false;
		}

        PropertyDefinition getConfig() {
			return null;
		}
    }

    //~---------------------------------------------< compatibility >

    public static NodeBuilder updateDefinition(NodeBuilder indexDefn){
		return null;
	}

    public static NodeBuilder updateDefinition(NodeBuilder indexDefn, String indexPath){
		return null;
	}

    /**
     * Constructs IndexingRule based on earlier format of index configuration
     */
    private static NodeBuilder createIndexRules(NodeState defn){
		return null;
	}

    private static NodeState getPropDefnNode(NodeState defn, String propName){
		return null;
	}

    //~---------------------------------------------< utility >

    private int determineMaxExtractLength() {
		return 0;
	}

    private NodeState getTikaConfigNode() {
		return null;
	}

    private Codec createCodec() {
		return null;
	}

    private static Set<String> getMultiProperty(NodeState definition, String propName){
		return null;
	}

    private static Set<String> toLowerCase(Set<String> values){
		return null;
	}

    private static List<String> getAllNodeTypes(ReadOnlyNodeTypeManager ntReg) {
		return null;
	}

    private static ReadOnlyNodeTypeManager createNodeTypeManager(final Tree root) {
		return null;
	}

    private static String getPrimaryTypeName(NodeState state) {
		return null;
	}

    private static Iterable<String> getMixinTypeNames(NodeState state) {
		return null;
	}

    private static boolean hasOrderableChildren(NodeState state){
		return false;
	}

    static int getSupportedTypes(NodeState defn, String typePropertyName, int defaultVal) {
		return 0;
	}

    static boolean includePropertyType(int includedPropertyTypes, int type){
		return false;
	}

    private static boolean hasFulltextEnabledIndexRule(List<IndexingRule> rules) {
		return false;
	}

    private static void markAsNtUnstructured(NodeBuilder nb){
	}

    private static IndexFormatVersion determineIndexFormatVersion(NodeState defn) {
		return null;
	}

    static IndexFormatVersion determineVersionForFreshIndex(NodeState defn){
		return null;
	}

    static IndexFormatVersion determineVersionForFreshIndex(NodeBuilder defnb){
		return null;
	}

    private static IndexFormatVersion determineVersionForFreshIndex(PropertyState fulltext,
                                                                    PropertyState compat,
                                                                    PropertyState version){
																		return null;
																	}

    private String[] getQueryPaths(NodeState defn) {
		return null;
	}

    private static IndexFormatVersion versionFrom(PropertyState ps){
		return null;
	}

    private static boolean hasIndexingRules(NodeState defn) {
		return false;
	}

    @CheckForNull
    private static String determineUniqueId(NodeState defn) {
		return null;
	}

    public boolean getActiveDeleteEnabled() {
		return false;
	}

    private static double getDefaultCostPerEntry(IndexFormatVersion version) {
		return 0.0d;
	}

    private static boolean supportsNRTIndexing(NodeState defn) {
		return false;
	}

    private static boolean supportsSyncIndexing(NodeState defn) {
		return false;
	}

    public static boolean supportsSyncOrNRTIndexing(NodeBuilder defn) {
		return false;
	}

    private static boolean supportsIndexingMode(NodeBuilder defn, String mode) {
		return false;
	}

    private static NodeState getIndexDefinitionState(NodeState defn) {
		return null;
	}

}
