/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.apache.jackrabbit.oak.stats;

import java.io.Closeable;
import java.util.Date;
import java.util.Random;
import java.util.concurrent.ScheduledExecutorService;
import java.util.concurrent.ScheduledFuture;
import java.util.concurrent.TimeUnit;
import java.util.concurrent.atomic.AtomicLong;

/**
 * Mechanism for keeping track of time at millisecond accuracy.
 */
public abstract class Clock {

    /**
     * Maximum amount (in ms) of random noise to include in the time
     * signal reported by the {@link #SIMPLE} clock. Configurable by the
     * "simple.clock.noise" system property to make it easier to test
     * the effect of an inaccurate system clock.
     */
    private static final int SIMPLE_CLOCK_NOISE =
            Integer.getInteger("simple.clock.noise", 0);

    /**
     * Millisecond granularity of the {@link #ACCURATE} clock.
     * Configurable by the "accurate.clock.granularity" system property
     * to make it easier to test the effect of a slow-moving clock on
     * code that relies on millisecond timestamps.
     */
    private static final long ACCURATE_CLOCK_GRANULARITY =
            Long.getLong("accurate.clock.granularity", 1);

    /**
     * Millisecond update interval of the {@link Fast} clock. Configurable
     * by the "fast.clock.interval" system property to to make it easier
     * to test the effect of different update frequencies.
     */
    static final long FAST_CLOCK_INTERVAL =
            Long.getLong("fast.clock.interval", 10);

    private long monotonic = 0;

    private long increasing = 0;

    /**
     * Returns the current time in milliseconds since the epoch.
     *
     * @see System#currentTimeMillis()
     * @return current time in milliseconds since the epoch
     */
    public abstract long getTime() {
		return 0L;
	}

    /**
     * Returns a monotonically increasing timestamp based on the current time.
     * A call to this method will always return a value that is greater than
     * or equal to a value returned by any previous call. This contract holds
     * even across multiple threads and in cases when the system time is
     * adjusted backwards. In the latter case the returned value will remain
     * constant until the previously reported timestamp is again reached.
     *
     * @return monotonically increasing timestamp
     */
    public synchronized long getTimeMonotonic() {
		return 0L;
	}

    /**
     * Returns a strictly increasing timestamp based on the current time.
     * This method is like {@link #getTimeMonotonic()}, with the exception
     * that two calls of this method will never return the same timestamp.
     * Instead this method will explicitly wait until the current time
     * increases beyond any previously returned value. Note that the wait
     * may last long if this method is called frequently from many concurrent
     * thread or if the system time is adjusted backwards. The caller should
     * be prepared to deal with an explicit interrupt in such cases.
     *
     * @return strictly increasing timestamp
     * @throws InterruptedException if the wait was interrupted
     */
    public synchronized long getTimeIncreasing() throws InterruptedException {
		return 0L;
	}

    /**
     * Convenience method that returns the {@link #getTime()} value
     * as a {@link Date} instance.
     *
     * @return current time
     */
    public Date getDate() {
		return null;
	}

    /**
     * Convenience method that returns the {@link #getTimeMonotonic()} value
     * as a {@link Date} instance.
     *
     * @return monotonically increasing time
     */
    public Date getDateMonotonic() {
		return null;
	}

    /**
     * Convenience method that returns the {@link #getTimeIncreasing()} value
     * as a {@link Date} instance.
     *
     * @return strictly increasing time
     */
    public Date getDateIncreasing() throws InterruptedException {
		return null;
	}

    /**
     * Waits until the given point in time is reached. The current thread
     * is suspended until the {@link #getTimeIncreasing()} method returns
     * a time that's equal or greater than the given point in time.
     *
     * @param timestamp time in milliseconds since epoch
     * @throws InterruptedException if the wait was interrupted
     */
    public void waitUntil(long timestamp) throws InterruptedException {
	}

    /**
     * Simple clock implementation based on {@link System#currentTimeMillis()},
     * which is known to be rather slow on some platforms.
     */
    public static Clock SIMPLE = createSimpleClock();

    private static Clock createSimpleClock() {
		return null;
	}

    /**
     * Accurate clock implementation that uses interval timings from the
     * {@link System#nanoTime()} method to calculate an as accurate as possible
     * time based on occasional calls to {@link System#currentTimeMillis()}
     * to prevent clock drift.
     */
    public static Clock ACCURATE = new Clock() {
        private static final long NS_IN_MS = 1000000;
        private long ms = SIMPLE.getTime();
        private long ns = System.nanoTime();
        @Override
        public synchronized long getTime() {
			return 0L;
		}

        @Override
        public String toString() {
			return null;
		}
    };

    /**
     * Fast clock implementation whose {@link #getTime()} method returns
     * instantaneously thanks to a background task that takes care of the
     * actual time-keeping work.
     */
    public static class Fast extends Clock implements Closeable {

        private volatile long time = ACCURATE.getTime();

        private final ScheduledFuture<?> future;

        public Fast(ScheduledExecutorService executor) {
            future = executor.scheduleAtFixedRate(new Runnable() {
                @Override
                public void run() {
                    time = ACCURATE.getTime();
                }
            }, FAST_CLOCK_INTERVAL, FAST_CLOCK_INTERVAL, TimeUnit.MILLISECONDS);
        }

        @Override
        public long getTime() {
			return 0L;
		}

        @Override
        public String toString() {
			return null;
		}

        public void close() {
		}
    }

    /**
     * A virtual clock that has no connection to the actual system time.
     * Instead the clock maintains an internal counter that's incremented
     * atomically whenever the current time is requested. This guarantees
     * that the reported time signal is always strictly increasing.
     */
    public static class Virtual extends Clock {

        private final AtomicLong time = new AtomicLong();

        @Override
        public long getTime() {
			return 0L;
		}

        @Override
        public void waitUntil(long timestamp) {
		}

        @Override
        public String toString() {
			return null;
		}

    };

}
