/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.apache.jackrabbit.spi.commons.identifier;

import org.apache.jackrabbit.spi.IdFactory;
import org.apache.jackrabbit.spi.PropertyId;
import org.apache.jackrabbit.spi.NodeId;
import org.apache.jackrabbit.spi.ItemId;
import org.apache.jackrabbit.spi.Name;
import org.apache.jackrabbit.spi.Path;
import org.apache.jackrabbit.spi.PathFactory;
import org.apache.jackrabbit.util.Text;

import javax.jcr.RepositoryException;
import java.io.Serializable;

/**
 * <code>AbstractIdFactory</code>...
 */
public abstract class AbstractIdFactory implements IdFactory {

    private static final char DELIMITER = '@';

    //----------------------------------------------------------< IdFactory >---
    /**
     * {@inheritDoc}
     * @see IdFactory#createNodeId(NodeId, Path)
     */
    public NodeId createNodeId(NodeId parentId, Path path) {
		return null;
	}

    /**
     * {@inheritDoc}
     * @see IdFactory#createNodeId(String, Path)
     */
    public NodeId createNodeId(String uniqueID, Path path) {
		return null;
	}

    /**
     * {@inheritDoc}
     * @see IdFactory#createNodeId(String)
     */
    public NodeId createNodeId(String uniqueID) {
		return null;
	}

    /**
     * {@inheritDoc}
     * @see IdFactory#createPropertyId(NodeId,Name)
     */
    public PropertyId createPropertyId(NodeId parentId, Name propertyName) {
		return null;
	}

    /**
     * @see IdFactory#toJcrIdentifier(NodeId)
     */
    public String toJcrIdentifier(NodeId nodeId) {
		return null;
	}

    /**
     * @see IdFactory#fromJcrIdentifier(String)
     */
    public NodeId fromJcrIdentifier(String jcrIdentifier) {
		return null;
	}

    //--------------------------------------------------------------------------
    /**
     * Subclassed need to define a PathFactory used to create IDs
     *
     * @return a implementation of <code>PathFactory</code>.
     */
    protected abstract PathFactory getPathFactory() {
		return null;
	}

    //------------------------------------------------------< Inner classes >---

    private static abstract class ItemIdImpl implements ItemId, Serializable {

        private final String uniqueID;
        private final Path path;

        private transient int hashCode = 0;

        private ItemIdImpl(String uniqueID, Path path) {
            if (uniqueID == null && path == null) {
                throw new IllegalArgumentException("Only uniqueID or relative path might be null.");
            }
            this.uniqueID = uniqueID;
            this.path = path;
        }

        private ItemIdImpl(NodeId parentId, Name name, PathFactory factory)
                throws RepositoryException {
            if (parentId == null || name == null) {
                throw new IllegalArgumentException("Invalid ItemIdImpl: parentId and name must not be null.");
            }
            this.uniqueID = parentId.getUniqueID();
            Path parentPath = parentId.getPath();
            if (parentPath != null) {
                this.path = factory.create(parentPath, name, true);
            } else {
                this.path = factory.create(name);
            }
        }

        public abstract boolean denotesNode() {
			return false;
		}

        public String getUniqueID() {
			return null;
		}

        public Path getPath() {
			return null;
		}

        /**
         * ItemIdImpl objects are equal if the have the same uuid and relative path.
         *
         * @param obj
         * @return
         */
        @Override
        public boolean equals(Object obj) {
			return false;
		}

        boolean equals(ItemId other) {
			return false;
		}

        /**
         * Returns the hash code of the uuid and the path. The computed hash code
         * is memorized for better performance.
         *
         * @return hash code
         * @see Object#hashCode()
         */
        @Override
        public int hashCode() {
			return 0;
		}

        /**
         * Combination of uuid and relative path
         *
         * @return
         */
        @Override
        public String toString() {
			return null;
		}
    }

    private static class NodeIdImpl extends ItemIdImpl implements NodeId {

        private static final long serialVersionUID = -360276648861146631L;

        public NodeIdImpl(String uniqueID) {
            super(uniqueID, null);
        }

        public NodeIdImpl(String uniqueID, Path path) {
            super(uniqueID, path);
        }

        public NodeIdImpl(NodeId parentId, Path path, PathFactory factory)
                throws RepositoryException {
            super(parentId.getUniqueID(), (parentId.getPath() != null) ? factory.create(parentId.getPath(), path, true) : path);
        }

        @Override
        public boolean denotesNode() {
			return false;
		}

        @Override
        public boolean equals(Object obj) {
			return false;
		}
    }

    private static class PropertyIdImpl extends ItemIdImpl implements PropertyId, Serializable {

        private static final long serialVersionUID = -1953124047770776444L;

        private final NodeId parentId;

        private PropertyIdImpl(NodeId parentId, Name name, PathFactory factory)
                throws RepositoryException {
            super(parentId, name, factory);
            this.parentId = parentId;
        }

        @Override
        public boolean denotesNode() {
			return false;
		}

        public NodeId getParentId() {
			return null;
		}

        public Name getName() {
			return null;
		}

        @Override
        public boolean equals(Object obj) {
			return false;
		}
    }
}
