/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.apache.jackrabbit.spi.commons.nodetype;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.Collections;
import java.util.Comparator;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;

import javax.jcr.PropertyType;

import org.apache.jackrabbit.spi.Name;
import org.apache.jackrabbit.spi.QItemDefinition;
import org.apache.jackrabbit.spi.QNodeDefinition;
import org.apache.jackrabbit.spi.QNodeTypeDefinition;
import org.apache.jackrabbit.spi.QPropertyDefinition;
import org.apache.jackrabbit.spi.QValueConstraint;
import org.apache.jackrabbit.spi.commons.name.NameConstants;

/**
 * A <code>NodeTypeDefDiff</code> represents the result of the comparison of
 * two node type definitions.
 * <p>
 * The result of the comparison can be categorized as one of the following types:
 * <p>
 * <b><code>NONE</code></b> indicates that there is no modification at all.
 * <p>
 * A <b><code>TRIVIAL</code></b> modification has no impact on the consistency
 * of existing content. The following modifications are considered
 * <code>TRIVIAL</code>:
 * <ul>
 * <li>changing node type <code>orderableChildNodes</code> flag
 * <li>changing node type <code>primaryItemName</code> value
 * <li>adding non-<code>mandatory</code> property/child node
 * <li>changing property/child node <code>protected</code> flag
 * <li>changing property/child node <code>onParentVersion</code> value
 * <li>changing property/child node <code>mandatory</code> flag to <code>false</code>
 * <li>changing property/child node <code>autoCreated</code> flag
 * <li>changing specific property/child node <code>name</code> to <code>*</code>
 * <li>changing child node <code>defaultPrimaryType</code>
 * <li>changing child node <code>sameNameSiblings</code> flag to <code>true</code>
 * <li>weaken child node <code>requiredPrimaryTypes</code> (e.g. by removing)
 * <li>weaken property <code>valueConstraints</code> (e.g. by removing a constraint
 * or by making a specific constraint less restrictive)
 * <li>changing property <code>defaultValues</code>
 * <li>changing specific property <code>requiredType</code> to <code>undefined</code>
 * <li>changing property <code>multiple</code> flag to <code>true</code>
 * </ul>
 * <p>
 * A <b><code>MAJOR</code></b> modification potentially <i>affects</i> the
 * consistency of existing content.
 *
 * All modifications that are not <b><code>TRIVIAL</code></b> are considered
 * <b><code>MAJOR</code></b>.
 *
 * @see #getType()
 */
public class NodeTypeDefDiff {

    /**
     * no modification
     */
    public static final int NONE = 0;
    /**
     * trivial modification: does not affect consistency of existing content
     */
    public static final int TRIVIAL = 1;
    /**
     * major modification: <i>does</i> affect consistency of existing content
     */
    public static final int MAJOR = 2;

    private final QNodeTypeDefinition oldDef;
    private final QNodeTypeDefinition newDef;
    private int type;

    private final List<PropDefDiff> propDefDiffs = new ArrayList<PropDefDiff>();
    private final List<ChildNodeDefDiff> childNodeDefDiffs = new ArrayList<ChildNodeDefDiff>();

    /**
     * Constructor
     * @param oldDef old definition
     * @param newDef new definition
     */
    private NodeTypeDefDiff(QNodeTypeDefinition oldDef, QNodeTypeDefinition newDef) {
        this.oldDef = oldDef;
        this.newDef = newDef;
        if (oldDef.equals(newDef)) {
            // definitions are identical
            type = NONE;
        } else {
            // definitions are not identical, determine type of modification

            // assume TRIVIAL change by default
            type = TRIVIAL;

            // check supertypes
            int tmpType = supertypesDiff();
            if (tmpType > type) {
                type = tmpType;
            }

            // check mixin flag (MAJOR modification)
            tmpType = mixinFlagDiff();
            if (tmpType > type) {
                type = tmpType;
            }

            // check abstract flag (MAJOR modification)
            tmpType = abstractFlagDiff();
            if (tmpType > type) {
                type = tmpType;
            }

            // no need to check orderableChildNodes flag (TRIVIAL modification)
            // no need to check queryable flag (TRIVIAL modification)

            // check property definitions
            PropDefDiffBuilder propDefDiffBuilder = new PropDefDiffBuilder(oldDef.getPropertyDefs(), newDef.getPropertyDefs());
            propDefDiffs.addAll(propDefDiffBuilder.getChildItemDefDiffs());
            tmpType = propDefDiffBuilder.getMaxType();
            if (tmpType > type) {
                type = tmpType;
            }

            // check child node definitions
            ChildNodeDefDiffBuilder childNodeDefDiffBuilder = new ChildNodeDefDiffBuilder(oldDef.getChildNodeDefs(), newDef.getChildNodeDefs());
            childNodeDefDiffs.addAll(childNodeDefDiffBuilder.getChildItemDefDiffs());
            tmpType = childNodeDefDiffBuilder.getMaxType();
            if (tmpType > type) {
                type = tmpType;
            }
        }
    }

    /**
     * @param oldDef old definition
     * @param newDef new definition
     * @return the diff
     */
    public static NodeTypeDefDiff create(QNodeTypeDefinition oldDef, QNodeTypeDefinition newDef) {
		return null;
	}

    /**
     * @return <code>true</code> if modified
     */
    public boolean isModified() {
		return false;
	}

    /**
     * @return <code>true</code> if trivial
     */
    public boolean isTrivial() {
		return false;
	}

    /**
     * @return <code>true</code> if major
     */
    public boolean isMajor() {
		return false;
	}

    /**
     * Returns the type of modification as expressed by the following constants:
     * <ul>
     * <li><b><code>NONE</code></b>: no modification at all
     * <li><b><code>TRIVIAL</code></b>: does not affect consistency of
     * existing content
     * <li><b><code>MAJOR</code></b>: <i>does</i> affect consistency of existing
     * content
     * </ul>
     *
     * @return the type of modification
     */
    public int getType() {
		return 0;
	}

    /**
     * @return <code>true</code> if mixin flag diff
     */
    public int mixinFlagDiff() {
		return 0;
	}

    /**
     * @return <code>true</code> if abstract flag diff
     */
    public int abstractFlagDiff() {
		return 0;
	}

    /**
     * @return <code>true</code> if supertypes diff
     */
    public int supertypesDiff() {
		return 0;
	}

    @Override
    public String toString() {
		return null;
	}

    private String toString(List<? extends ChildItemDefDiff> childItemDefDiffs) {
		return null;
	}

    private String modificationTypeToString(int modificationType) {
		return null;
	}


    //--------------------------------------------------------< inner classes >

    private abstract class ChildItemDefDiffBuilder<T extends QItemDefinition, V extends ChildItemDefDiff<T>> {

        private final List<V> childItemDefDiffs = new ArrayList<V>();

        private ChildItemDefDiffBuilder(T[] oldDefs, T[] newDefs) {
            buildChildItemDefDiffs(collectChildNodeDefs(oldDefs), collectChildNodeDefs(newDefs));
        }

        private void buildChildItemDefDiffs(Map<Object, List<T>> oldDefs, Map<Object, List<T>> newDefs) {
		}

        private Map<Object, List<T>> collectChildNodeDefs(final T[] defs) {
			return null;
		}

        abstract Object createQItemDefinitionId(T def) {
			return null;
		}

        abstract V createChildItemDefDiff(T def1, T def2) {
			return null;
		}

        Collection<V> getChildItemDefDiffs(List<T> defs1, List<T> defs2) {
			return null;
		}

        private boolean alreadyMatched(final List<V> result, final T newDef, final T oldDef, final int allowedNewNull, final int allowedOldNull) {
			return false;
		}

        List<V> getChildItemDefDiffs() {
			return null;
		}

        int getMaxType() {
			return 0;
		}
    }

    private abstract class ChildItemDefDiff<T extends QItemDefinition> {
        protected final T oldDef;
        protected final T newDef;
        protected int type;

        private ChildItemDefDiff(T oldDef, T newDef) {
            this.oldDef = oldDef;
            this.newDef = newDef;
            init();
        }

        protected void init() {
		}

        T getOldDef() {
			return null;
		}

        T getNewDef() {
			return null;
		}

        int getType() {
			return 0;
		}

        boolean isAdded() {
			return false;
		}

        boolean isRemoved() {
			return false;
		}

        boolean isModified() {
			return false;
		}

        @Override
        public String toString() {
			return null;
		}

    }

    private class PropDefDiff extends ChildItemDefDiff<QPropertyDefinition> {

        private PropDefDiff(QPropertyDefinition oldDef, QPropertyDefinition newDef) {
            super(oldDef, newDef);
        }

        @Override
        protected void init() {
		}
    }

    private class ChildNodeDefDiff extends ChildItemDefDiff<QNodeDefinition> {

        private ChildNodeDefDiff(QNodeDefinition oldDef, QNodeDefinition newDef) {
            super(oldDef, newDef);
        }

        @Override
        protected void init() {
		}
    }

    /**
     * Identifier used to identify corresponding property definitions
     */
    private static class QPropertyDefinitionId {

        private Name declaringNodeType;
        private Name name;

        private QPropertyDefinitionId(QPropertyDefinition def) {
            declaringNodeType = def.getDeclaringNodeType();
            name = def.getName();
        }

        //---------------------------------------< java.lang.Object overrides >
        @Override
        public boolean equals(Object obj) {
			return false;
		}

        @Override
        public int hashCode() {
			return 0;
		}
    }

    /**
     * Identifier used to identify corresponding node definitions
     */
    private static class QNodeDefinitionId {

        private Name declaringNodeType;
        private Name name;

        private QNodeDefinitionId(QNodeDefinition def) {
            declaringNodeType = def.getDeclaringNodeType();
            name = def.getName();
        }

        //---------------------------------------< java.lang.Object overrides >
        @Override
        public boolean equals(Object obj) {
			return false;
		}

        @Override
        public int hashCode() {
			return 0;
		}
    }

    private class ChildNodeDefDiffBuilder extends ChildItemDefDiffBuilder<QNodeDefinition, ChildNodeDefDiff> {

        private ChildNodeDefDiffBuilder(final QNodeDefinition[] defs1, final QNodeDefinition[] defs2) {
            super(defs1, defs2);
        }

        @Override
        Object createQItemDefinitionId(final QNodeDefinition def) {
			return null;
		}

        @Override
        ChildNodeDefDiff createChildItemDefDiff(final QNodeDefinition def1, final QNodeDefinition def2) {
			return null;
		}
    }

    private class PropDefDiffBuilder extends ChildItemDefDiffBuilder<QPropertyDefinition, PropDefDiff> {

        private PropDefDiffBuilder(final QPropertyDefinition[] defs1, final QPropertyDefinition[] defs2) {
            super(defs1, defs2);
        }

        @Override
        Object createQItemDefinitionId(final QPropertyDefinition def) {
			return null;
		}

        @Override
        PropDefDiff createChildItemDefDiff(final QPropertyDefinition def1, final QPropertyDefinition def2) {
			return null;
		}
    }

}
