/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

// Sep 14, 2000:
//  Fixed problem with namespace handling. Contributed by
//  David Blondeau <blondeau@intalio.com>
// Sep 14, 2000:
//  Fixed serializer to report IO exception directly, instead at
//  the end of document processing.
//  Reported by Patrick Higgins <phiggins@transzap.com>
// Aug 21, 2000:
//  Fixed bug in startDocument not calling prepare.
//  Reported by Mikael Staldal <d96-mst-ingen-reklam@d.kth.se>
// Aug 21, 2000:
//  Added ability to omit DOCTYPE declaration.


package org.apache.jackrabbit.vault.util.xml.serialize;


import java.io.IOException;
import java.io.OutputStream;
import java.io.Writer;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.TreeMap;

import org.apache.jackrabbit.vault.util.xml.xerces.dom.DOMMessageFormatter;
import org.apache.jackrabbit.vault.util.xml.xerces.util.NamespaceSupport;
import org.apache.jackrabbit.vault.util.xml.xerces.util.SymbolTable;
import org.apache.jackrabbit.vault.util.xml.xerces.util.XMLChar;
import org.apache.jackrabbit.vault.util.xml.xerces.util.XMLSymbols;
import org.apache.jackrabbit.vault.util.xml.xerces.xni.NamespaceContext;
import org.w3c.dom.Attr;
import org.w3c.dom.DOMError;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;
import org.xml.sax.AttributeList;
import org.xml.sax.Attributes;
import org.xml.sax.SAXException;
import org.xml.sax.helpers.AttributeListImpl;
import org.xml.sax.helpers.AttributesImpl;

/**
 * <b>ASF Note<b>: This class and dependencies were copied from the sun jdk1.5
 * source base. The original serializer is extended by a new OutputFormat
 * setting that allows to break the line for each attribute. additionally,
 * all attribute names are sorted alphabetically.
 * Some Features were removed to limit the number of dependent classes:
 * <ul>
 * <li> dom filter support
 * <li> all text nodes as CDATA feature
 * <li> skip attribute default values feature
 * <li> entity node reference feature
 * </ul>
 * <p>
 *  
 * Implements an XML serializer supporting both DOM and SAX pretty
 * serializing. For usage instructions see {@link Serializer}.
 * <p>
 * If an output stream is used, the encoding is taken from the
 * output format (defaults to <tt>UTF-8</tt>). If a writer is
 * used, make sure the writer uses the same encoding (if applies)
 * as specified in the output format.
 * <p>
 * The serializer supports both DOM and SAX. SAX serializing is done by firing
 * SAX events and using the serializer as a document handler. DOM serializing is done
 * by calling {@link #serialize(Document)} or by using DOM Level 3
 * <p>
 * If an I/O exception occurs while serializing, the serializer
 * will not throw an exception directly, but only throw it
 * at the end of serializing (either DOM or SAX's {@link
 * org.xml.sax.DocumentHandler#endDocument}.
 * <p>
 * For elements that are not specified as whitespace preserving,
 * the serializer will potentially break long text lines at space
 * boundaries, indent lines, and serialize elements on separate
 * lines. Line terminators will be regarded as spaces, and
 * spaces at beginning of line will be stripped.
 *
 * @author <a href="mailto:arkin@intalio.com">Assaf Arkin</a>
 * @author <a href="mailto:rahul.srivastava@sun.com">Rahul Srivastava</a>
 * @author Elena Litani IBM
 * @see Serializer
 */
public class XMLSerializer extends BaseMarkupSerializer {

    //
    // constants
    //

    protected static final boolean DEBUG = false;

    //
    // data
    //

    //
    // DOM Level 3 implementation: variables initialized in DOMSerializerImpl
    //

    /**
     * stores namespaces in scope
     */
    protected NamespaceSupport fNSBinder;

    /**
     * stores all namespace bindings on the current element
     */
    protected NamespaceSupport fLocalNSBinder;

    /**
     * symbol table for serialization
     */
    protected SymbolTable fSymbolTable;

    protected final static String PREFIX = "NS";

    /**
     * Controls whether namespace fixup should be performed during
     * the serialization.
     * NOTE: if this field is set to true the following
     * fields need to be initialized: fNSBinder, fLocalNSBinder, fSymbolTable,
     * XMLSymbols.EMPTY_STRING, fXmlSymbol, fXmlnsSymbol
     */
    protected boolean fNamespaces = false;


    private boolean fPreserveSpace;


    /**
     * Constructs a new serializer. The serializer cannot be used without
     * calling {@link #setOutputCharStream} or {@link #setOutputByteStream}
     * first.
     */
    public XMLSerializer() {
        super(new OutputFormat(Method.XML, null, false));
    }


    /**
     * Constructs a new serializer. The serializer cannot be used without
     * calling {@link #setOutputCharStream} or {@link #setOutputByteStream}
     * first.
     */
    public XMLSerializer(OutputFormat format) {
        super(format != null ? format : new OutputFormat(Method.XML, null, false));
        _format.setMethod(Method.XML);
    }


    /**
     * Constructs a new serializer that writes to the specified writer
     * using the specified output format. If <tt>format</tt> is null,
     * will use a default output format.
     *
     * @param writer The writer to use
     * @param format The output format to use, null for the default
     */
    public XMLSerializer(Writer writer, OutputFormat format) {
        super(format != null ? format : new OutputFormat(Method.XML, null, false));
        _format.setMethod(Method.XML);
        setOutputCharStream(writer);
    }


    /**
     * Constructs a new serializer that writes to the specified output
     * stream using the specified output format. If <tt>format</tt>
     * is null, will use a default output format.
     *
     * @param output The output stream to use
     * @param format The output format to use, null for the default
     */
    public XMLSerializer(OutputStream output, OutputFormat format) {
        super(format != null ? format : new OutputFormat(Method.XML, null, false));
        _format.setMethod(Method.XML);
        setOutputByteStream(output);
    }


    public void setOutputFormat(OutputFormat format) {
	}


    /**
     * This methods turns on namespace fixup algorithm during
     * DOM serialization.
     *
     * @param namespaces
     */
    public void setNamespaces(boolean namespaces) {
	}

    //-----------------------------------------//
    // SAX content handler serializing methods //
    //-----------------------------------------//


    public void startElement(String namespaceURI, String localName,
                             String rawName, Attributes attrs)
            throws SAXException {
			}


    public void endElement(String namespaceURI, String localName,
                           String rawName)
            throws SAXException {
			}


    public void endElementIO(String namespaceURI, String localName,
                             String rawName)
            throws IOException {
			}

    //------------------------------------------//
    // SAX document handler serializing methods //
    //------------------------------------------//


    public void startElement(String tagName, AttributeList attrs)
            throws SAXException {
			}


    public void endElement(String tagName)
            throws SAXException {
			}

    //------------------------------------------//
    // Generic node serializing methods methods //
    //------------------------------------------//


    /**
     * Called to serialize the document's DOCTYPE by the root element.
     * The document type declaration must name the root element,
     * but the root element is only known when that element is serialized,
     * and not at the start of the document.
     * <p>
     * This method will check if it has not been called before ({@link #_started}),
     * will serialize the document type declaration, and will serialize all
     * pre-root comments and PIs that were accumulated in the document
     * (see {@link #serializePreRoot}). Pre-root will be serialized even if
     * this is not the first root element of the document.
     */
    protected void startDocument(String rootTagName)
            throws IOException {
			}


    /**
     * Called to serialize a DOM element. Equivalent to calling {@link
     * #startElement}, {@link #endElement} and serializing everything
     * inbetween, but better optimized.
     */
    protected void serializeElement(Element elem)
            throws IOException {
			}


    /**
     * Serializes a namespace attribute with the given prefix and value for URI.
     * In case prefix is empty will serialize default namespace declaration.
     *
     * @param prefix
     * @param uri
     * @throws IOException
     */

    private void printNamespaceAttr(String prefix, String uri) throws IOException {
	}

    private Iterable<Node> sortAttributes(final NamedNodeMap attributeList) {
		return null;
	}

    private Attributes sortAttributes(Attributes attributeList) {
		return null;
	}

    private AttributeList sortAttributes(AttributeList attributeList) {
		return null;
	}

    /**
     * Prints attribute.
     * NOTE: xml:space attribute modifies output format
     *
     * @param name
     * @param value
     * @param isSpecified
     * @throws IOException
     */
    private void printAttribute(String name, String value, boolean isSpecified, Attr attr) throws IOException {
	}

    protected String getEntityRef(int ch) {
		return null;
	}


    /**
     * Retrieve and remove the namespaces declarations from the list of attributes.
     */
    private Attributes extractNamespaces(Attributes attrs)
            throws SAXException {
				return null;
			}

    //
    // Printing attribute value
    //
    protected void printEscaped(String source) throws IOException {
	}

    /**
     * print text data
     */
    protected void printXMLChar(int ch) throws IOException {
	}

    protected void printText(String text, boolean preserveSpace, boolean unescaped)
            throws IOException {
			}


    protected void printText(char[] chars, int start, int length,
                             boolean preserveSpace, boolean unescaped) throws IOException {
							}


    /**
     * DOM Level 3:
     * Check a node to determine if it contains unbound namespace prefixes.
     *
     * @param node The node to check for unbound namespace prefices
     */
    protected void checkUnboundNamespacePrefixedNode(Node node) throws IOException {
	}

    public boolean reset() {
		return false;
	}

    private static class NamedNodeMapIterator implements Iterator<Node> {

        private final NamedNodeMap nodeMap;
        private int index = 0;
        private final int length;

        NamedNodeMapIterator(NamedNodeMap nodeMap) {
            this.nodeMap = nodeMap;
            this.length = nodeMap.getLength();
        }

        @Override public boolean hasNext() {
			return false;
		}

        @Override public Node next() {
			return null;
		}

        @Override public void remove() {
		}
    }
}




